@@ -1,6 +1,6 @@
 /*
    Egon Animator
-   Copyright (C) 1997-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 #include <unistd.h>
 
 #include "../common/common.h"
-#include "../common/cmalloc.h"
+#include <Mowitz/MwUtils.h>
 #include "../common/plugin.h"
 #include "../siod/siod.h"
 
@@ -42,10 +42,18 @@
 
 typedef struct {
 	char *name;
-	 LISP(*function) ();
-} s_fn_table;
+	LISP(*function) (void);
+} s_fn_table0;
 
-extern s_fn_table fn_table[];
+typedef struct {
+	char *name;
+	LISP(*function) (LISP);
+} s_fn_table1;
+
+typedef struct {
+	char *name;
+	LISP(*function) (LISP, LISP);
+} s_fn_table2;
 
 /* moving around */
 
@@ -75,23 +83,9 @@ static LISP what_cursor_position(void)
 
 /* ---
 */
-static LISP spawn(LISP command)
+static LISP lspawn(LISP command)
 {
-	char *argv[20], *p, cmd[1024];
-	int argc = 0;
-
-	strcpy(cmd, get_c_string(command));
-	for (p = strtok(cmd, " \t\r\n");
-		p && argc < 20;
-		p = strtok(NULL, " \t\r\n")) {
-		argv[argc++] = p;
-	}
-	argv[argc] = NULL;
-	if (!fork()) {
-		/* this is the child */
-		execvp(argv[0], argv);
-		exit(0);
-	}
+	MwSpawn(get_c_string(command));
 	return NIL;
 }
 
@@ -107,7 +101,7 @@ static LISP ask_for_file(void)
 	if (path[0] == '\0')
 		getcwd(path, 1024);
 	name[0] = fn[0] = '\0';
-	if (!select_file(path, name, NULL, fmt))
+	if (!select_file(path, name, NULL, fmt, 0))
 		return NIL;
 	sprintf(fn, "%s/%s", path, name);
 	return strcons(strlen(fn), fn);
@@ -131,7 +125,7 @@ static LISP load_buffer(void)
 	}
 	strcpy(name, "");
 	fn[0] = '\0';
-	if (select_file(path, name, loader_patterns, fmt)) {
+	if (select_file(path, name, loader_patterns, fmt, 0)) {
 		sprintf(fn, "%s/%s", path, name);
 		b = new_buffer(buffer_name(fn), fn);
 		llpr("Loading");
@@ -146,6 +140,7 @@ static LISP load_buffer(void)
 
 		b->change = FALSE;
 		w_list->buf = b;
+		w_list->sht = 0;
 		pr_scr_flag = TRUE;
 	}
 	activate_window(w_list);
@@ -156,7 +151,7 @@ static LISP load_buffer(void)
 Load using an external program
 
    1. Ask for the program to use
-   2. Run the program and save output to file in /tmp
+   2. Run the program and save output to file in siag_tmpdir
    3. Load the file using NULL format (i.e. ask for type)
 */
 
@@ -182,8 +177,8 @@ static LISP load_external(void)
 		while (!feof(fp) && loaders < 20) {
 			fgets(b, 250, fp);
 			if ((p = strtok(b, ":")) && (q = strtok(NULL, "\n"))) {
-				loadname[loaders] = cstrdup(p);
-				loadprog[loaders] = cstrdup(q);
+				loadname[loaders] = MwStrdup(p);
+				loadprog[loaders] = MwStrdup(q);
 				loaders++;
 			}
 		}
@@ -194,7 +189,7 @@ static LISP load_external(void)
 	i = select_from_list("External Program:", loadname, loaders);
 
 	if (i >= 0 && ask_for_str("Parameters:", param)) {
-		sprintf(fn, "/tmp/egon%ld", (long)getpid());
+		sprintf(fn, "%s/egon%ld", siag_tmpdir, (long)getpid());
 		sprintf(cmd, "%s %s > %s", loadprog[i], param, fn);
 		if (system(cmd)) {
 			llpr("External program failed");
@@ -209,6 +204,7 @@ static LISP load_external(void)
 
 		b->change = FALSE;
 		w_list->buf = b;
+		w_list->sht = 0;
 		pr_scr_flag = TRUE;
 	}
 	activate_window(w_list);
@@ -273,6 +269,29 @@ static LISP save_buffer(void)
 	return NIL;
 }
 
+static LISP lsavematrix(LISP path, LISP bname, LISP format)
+{
+	char *p, *fmt;
+	buffer *b;
+
+	if (NULLP(bname)) b = w_list->buf;
+	else b = find_buffer_by_name(get_c_string(bname));
+	if (!b) {
+		llpr("No such buffer");
+		return NIL;
+	}
+	p = get_c_string(path);
+	if (NULLP(format)) fmt = guess_file_format(p);
+	else fmt = get_c_string(format);
+
+	if (savematrix(p, b, fmt)) {
+		llpr("File saved");
+		return NIL;
+	} else {
+		return a_true_value();
+	}
+}
+
 /* ---
    Save the buffer in the currently active window to a named file.
 */
@@ -293,7 +312,7 @@ static LISP save_buffer_as(void)
 	if (p) strcpy(name, p+1);
 	else strcpy(name, w_list->buf->path);
 	fn[0] = '\0';
-	if (select_file(path, name, saver_patterns, fmt)) {
+	if (select_file(path, name, saver_patterns, fmt, 1)) {
 		sprintf(fn, "%s/%s", path, name);
 		llpr("Saving");
 		if (savematrix(fn, w_list->buf, fmt))
@@ -311,7 +330,7 @@ static LISP save_buffer_as(void)
 Save using an external program
 
    1. Ask for the program to use
-   2. Save to a file in /tmp using NULL format (i.e. ask for type)
+   2. Save to a file in siag_tmpdir using NULL format (i.e. ask for type)
    3. Run the program and read the file as input
 */
 
@@ -336,8 +355,8 @@ static LISP save_external(void)
 		while (!feof(fp) && savers < 20) {
 			fgets(b, 250, fp);
 			if ((p = strtok(b, ":")) && (q = strtok(NULL, "\n"))) {
-				savename[savers] = cstrdup(p);
-				saveprog[savers] = cstrdup(q);
+				savename[savers] = MwStrdup(p);
+				saveprog[savers] = MwStrdup(q);
 				savers++;
 			}
 		}
@@ -351,7 +370,7 @@ static LISP save_external(void)
 	if (i >= 0 && ask_for_str("Parameters:", param)) {
 		llpr("Saving");
 
-		sprintf(fn, "/tmp/egon%ld", (long)getpid());
+		sprintf(fn, "%s/egon%ld", siag_tmpdir, (long)getpid());
 		if (savematrix(fn, w_list->buf, NULL)) {
 			error_box("Couldn't save");
 			return NIL;
@@ -417,24 +436,12 @@ static LISP kill_buffer(void)
 	return NIL;
 }
 
-/* ---
-*/
-static LISP preview_block(void)
-{
-	preview();
-	return NIL;
-}
 
-/* ---
-*/
-static LISP print_block(void)
-{
-	int nr, nk;
+extern char *psformat;
 
-	nr = 0;
-	nk = 0;
-	printer();
-	return NIL;
+static LISP lpsformat(void)
+{
+	return strcons(strlen(psformat), psformat);
 }
 
 /* ---
@@ -464,17 +471,34 @@ static LISP lalertbox(LISP prompt, LISP buttons)
 
 	while (bno < 10 && NNULLP(buttons)) {
 		char *c = get_c_string(car(buttons));
-		if (c) btext[bno] = cstrdup(c);
-		else btext[bno] = cstrdup("button");
+		if (c) btext[bno] = MwStrdup(c);
+		else btext[bno] = MwStrdup("button");
 		buttons = cdr(buttons);
 		bno++;
 	}
 	if (p == NULL) p = "prompt";
 	n = alert_box(p, btext, bno);
-	while (bno) cfree(btext[--bno]);
+	while (bno) MwFree(btext[--bno]);
 	return flocons(n);
 }
 
+static LISP laboutbox(void)
+{
+        char msg[1024];
+        sprintf(msg, "%s\n\n%s\n\nUlric Eriksson, ulric@siag.nu\n\n%s\n",
+		version,
+		_("An animation program for X"),
+		_("Part of Siag Office"));
+        about_box(msg);
+        return NIL;
+}
+ 
+static LISP laboutsiag(void)
+{
+        about_siag();
+        return NIL;
+}
+
 /* ---
 */
 static LISP quit_egon(void)
@@ -547,16 +571,6 @@ static LISP get_format(void)
 /* These functions allow implementation of commands in Scheme
 	rather than in C with Scheme wrappers */
 
-/* ---
-*/
-static LISP bell(void)
-{
-#if 0
-	XBell(XtDisplay(pLevel), 100);
-#endif
-	return NIL;
-}
-
 /* ---
 */
 static LISP lask_for_str(LISP prompt, LISP buf)
@@ -633,13 +647,15 @@ static LISP ani_c_test(void)
 static LISP ani_begin(void)
 {
 	buffer *b = w_list->buf;
+	int sht = w_list->sht;
+	sheet *s = &b->sht[sht];
 	/* should free old objects first */
-	b->cast = NULL;
-	b->delta = 100;
-	b->duration = 1000;
+	s->cast = NULL;
+	s->delta = 100;
+	s->duration = 1000;
 	b->width = 600;
 	b->height = 400;
-	b->bg = NULL;
+	s->bg = NULL;
 	b->change = TRUE;
 	w_list->object = NULL;
 	w_list->script = NULL;
@@ -654,7 +670,8 @@ static LISP select_object(LISP name)
 {
 	char *string = get_c_string(name);
 
-	ani_object *o = w_list->buf->cast;
+	int sht = w_list->sht;
+	MwAniObject *o = w_list->buf->sht[sht].cast;
 
 	while (o) {
 		if (!strcmp(o->name, string)) break;
@@ -674,7 +691,7 @@ static LISP select_object(LISP name)
 */
 static LISP select_tick(LISP tick)
 {
-	ani_script *s;
+	MwAniScript *s;
 	int t = get_c_long(tick);
 	if (!w_list->object) {
 		llpr("No objects!");
@@ -699,22 +716,23 @@ static LISP select_tick(LISP tick)
 static LISP lani_object(LISP obj)
 {
 	buffer *b = w_list->buf;
-	ani_object *lasto = w_list->object, *newo;
-	ani_script *lasts;
+	MwAniObject *lasto = w_list->object, *newo;
+	MwAniScript *lasts;
 	int t = get_c_long(obj);
+	int sht = w_list->sht;
 
-	newo = (ani_object *)cmalloc(sizeof(ani_object));
+	newo = (MwAniObject *)MwMalloc(sizeof(MwAniObject));
 	if (lasto) {	/* put newo after lasto */
 		newo->next = lasto->next;
 		lasto->next = newo;
 	} else {	/* put newo first in object list */
-		b->cast = newo;
+		b->sht[sht].cast = newo;
 		newo->next = NULL;
 	}
 	w_list->object = newo;
 	newo->type = t;
 	newo->name = type2name(t);
-	lasts = newo->script = (ani_script *)cmalloc(sizeof(ani_script));
+	lasts = newo->script = (MwAniScript *)MwMalloc(sizeof(MwAniScript));
 	lasts->x = 0;
 	lasts->y = 0;
 	lasts->width = 0;
@@ -740,7 +758,7 @@ static LISP set_type(LISP type)
 		llpr("No object selected");
 	} else {
 		w_list->object->type = t;
-		cfree(w_list->object->name);
+		MwFree(w_list->object->name);
 		w_list->object->name = type2name(t);
 	}
 	w_list->buf->change = TRUE;
@@ -755,15 +773,16 @@ FIXME: these two should free the memory used
 static LISP del_object(void)
 {
 	buffer *b = w_list->buf;
-	ani_object *o = w_list->object;
-	ani_object *o1 = w_list->buf->cast;
+	int sht = w_list->sht;
+	MwAniObject *o = w_list->object;
+	MwAniObject *o1 = w_list->buf->sht[sht].cast;
 
 	if (!o) {
 		llpr("None selected");
 		return NIL;
 	}
 	if (o == o1) {	/* first object */
-		w_list->buf->cast = o->next;
+		w_list->buf->sht[sht].cast = o->next;
 		w_list->object = o->next;
 	} else {	/* some other object; find the one before */
 		while (o1->next != o) o1 = o1->next;
@@ -784,8 +803,8 @@ static LISP del_object(void)
 static LISP del_time(void)
 {
 	buffer *b = w_list->buf;
-	ani_object *o = w_list->object;
-	ani_script *s, *s1;
+	MwAniObject *o = w_list->object;
+	MwAniScript *s, *s1;
 
 	if (!o) {
 		llpr("None selected");
@@ -810,19 +829,49 @@ static LISP del_time(void)
 static LISP ani_background(LISP bgp)
 {
 	buffer *b = w_list->buf;
-	if (b->bg) cfree(b->bg);
-	b->bg = cstrdup(get_c_string(bgp));
+	int sht = w_list->sht;
+	if (b->sht[sht].bg) MwFree(b->sht[sht].bg);
+	if (NULLP(bgp)) {
+		b->sht[sht].bg = NULL;
+	} else {
+		b->sht[sht].bg = MwStrdup(get_c_string(bgp));
+	}
 	b->change = TRUE;
 	pr_scr_flag = TRUE;
 	return NIL;
 }
 
+/* ---
+*/
+static LISP ani_bgrad(LISP bgp)
+{
+	buffer *b = w_list->buf;
+	int sht = w_list->sht;
+	if (b->sht[sht].bgrad) MwFree(b->sht[sht].bgrad);
+	if (NULLP(bgp)) {
+		b->sht[sht].bgrad = NULL;
+	} else {
+		b->sht[sht].bgrad = MwStrdup(get_c_string(bgp));
+	}
+	b->change = TRUE;
+	pr_scr_flag = TRUE;
+	return NIL;
+}
+
+static LISP get_bgrad(void)
+{
+	buffer *b = w_list->buf;
+	int sht = w_list->sht;
+	if (b->sht[sht].bgrad) return strcons(-1, b->sht[sht].bgrad);
+	return strcons(-1, "");
+}
+
 /* ---
 */
 static LISP ani_time(LISP tim)
 {
-	ani_object *o = w_list->object;
-	ani_script *s, *news;
+	MwAniObject *o = w_list->object;
+	MwAniScript *s, *news;
 	unsigned int t = get_c_long(tim);
 	
 	if (!o) return NIL;
@@ -835,7 +884,7 @@ static LISP ani_time(LISP tim)
 	if (s->time == t) return NIL;	/* dup tick */
 
 	/* simply copy the last tick, but with a new time */
-	news = (ani_script *)cmalloc(sizeof(ani_script));
+	news = (MwAniScript *)MwMalloc(sizeof(MwAniScript));
 	*news = *s;
 	news->next = s->next;
 	news->time = t;
@@ -850,7 +899,8 @@ static LISP ani_time(LISP tim)
 static LISP lani_duration(LISP tim)
 {
 	buffer *b = w_list->buf;
-	b->duration = get_c_long(tim);
+	int sht = w_list->sht;
+	b->sht[sht].duration = get_c_long(tim);
 	pr_scr_flag = TRUE;
 	return NIL;
 }
@@ -860,7 +910,8 @@ static LISP lani_duration(LISP tim)
 static LISP lani_delta(LISP tim)
 {
 	buffer *b = w_list->buf;
-	b->delta = get_c_long(tim);
+	int sht = w_list->sht;
+	b->sht[sht].delta = get_c_long(tim);
 	pr_scr_flag = TRUE;
 	return NIL;
 }
@@ -888,59 +939,78 @@ static LISP ani_end(void)
 	return NIL;
 }
 
-
-/* ---
-*/
-static LISP lregister_x_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
+static LISP ladd_sheet(void)
 {
-	return flocons(register_x_font(get_c_string(name),
-					get_c_string(x),
-					get_c_string(xi),
-					get_c_string(xb),
-					get_c_string(xbi)));
+	buffer *b = w_list->buf;
+	int n = b->nsht;
+	int m = buffer_add_sheet(b, n);
+	pr_scr_flag = 1;
+	b->change = 1;
+	return flocons(m);
 }
 
-/* ---
-*/
-static LISP lregister_ps_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
+static LISP lremove_sheet(void)
 {
-	return flocons(register_ps_font(get_c_string(name),
-					get_c_string(x),
-					get_c_string(xi),
-					get_c_string(xb),
-					get_c_string(xbi)));
+	buffer *b = w_list->buf;
+	int m;
+	pr_scr_flag = 1;
+	b->change = 1;
+	m = buffer_remove_sheet(b, w_list->sht);
+	if (w_list->sht >= m) {
+		w_list->sht = m-1;
+	}
+	return flocons(m);
 }
 
-/* ---
-*/
-static LISP lregister_t1_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
+static LISP lrename_sheet(void)
 {
-	return flocons(register_t1_font(get_c_string(name),
-					get_c_string(x),
-					get_c_string(xi),
-					get_c_string(xb),
-					get_c_string(xbi)));
+	char name[1000];
+	buffer *b = w_list->buf;
+	strcpy(name, b->sht[w_list->sht].name);
+	if (ask_for_str("Name: ", name)) {
+		buffer_rename_sheet(b, w_list->sht, name);
+		pr_scr_flag = 1;
+		b->change = 1;
+	}
+	return NIL;
 }
 
-/* ---
-*/
-static LISP lfont_alias(LISP alias, LISP name)
+static LISP lmove_sheet(LISP f, LISP t)
 {
-	return flocons(font_alias(get_c_string(alias),
-				get_c_string(name)));
+	int from = get_c_long(f);
+	int to = get_c_long(t);
+	buffer *b = w_list->buf;
+	int nsht = b->nsht;
+	sheet sht;
+	int i;
+
+	if (from < 0 || from >= nsht || to < -1 || to >= nsht)
+		return NIL;
+
+	if (to == -1) to = nsht-1;
+
+	sht = b->sht[from];
+
+	for (i = from; i < nsht-1; i++)
+		b->sht[i] = b->sht[i+1];
+
+	for (i = nsht-1; i > to; i--)
+		b->sht[i] = b->sht[i-1];
+
+	b->sht[to] = sht;
+
+	w_list->sht = to;
+
+	pr_scr_flag = 1;
+	b->change = 1;
+	return NIL;
 }
 
-/* ---
-*/
-static LISP lregister_color(LISP name, LISP r, LISP g, LISP b)
+static LISP lget_sheet(void)
 {
-	return flocons(register_color(get_c_string(name),
-					get_c_long(r),
-					get_c_long(g),
-					get_c_long(b)));
+	return flocons(w_list->sht);
 }
 
-
 /* ---
 */
 static LISP labort(void)
@@ -956,44 +1026,44 @@ and turns it into an index into the format table.
 */
 static LISP lencode_format(LISP f)
 {
-	sfmt fmt;
+	MwFmt fmt;
 	int result;
 
-	decode_format(0, ~0, &fmt);	/* put valid fields in fmt */
+	MwDecodeFormat(0, ~0, &fmt);	/* put valid fields in fmt */
 	while (NNULLP(f)) {
 		char *name = get_c_string(CAR(CAR(f)));
-		int mask = fmt_attr2mask(name);
+		int mask = MwFmtAttrToMask(name);
 		LISP value = CDR(CAR(f));
 		f = CDR(f);
 		switch (mask) {
-		case SFMT_FAMILY:
+		case MW_FMT_FAMILY:
 			fmt.family = get_c_string(value);
 			break;
-		case SFMT_SIZE:
+		case MW_FMT_SIZE:
 			fmt.size = get_c_long(value);
 			break;
-		case SFMT_FG:
+		case MW_FMT_FG:
 			fmt.fg = get_c_string(value);
 			break;
-		case SFMT_BG:
+		case MW_FMT_BG:
 			fmt.bg = get_c_string(value);
 			break;
-		case SFMT_BOLD:
+		case MW_FMT_BOLD:
 			fmt.bold = get_c_long(value);
 			break;
-		case SFMT_ITALIC:
+		case MW_FMT_ITALIC:
 			fmt.italic = get_c_long(value);
 			break;
-		case SFMT_ULINE:
+		case MW_FMT_ULINE:
 			fmt.uline = get_c_long(value);
 			break;
-		case SFMT_HADJ:
+		case MW_FMT_HADJ:
 			fmt.hadj = get_c_long(value);
 			break;
-		case SFMT_VADJ:
+		case MW_FMT_VADJ:
 			fmt.vadj = get_c_long(value);
 			break;
-		case SFMT_STYLE:
+		case MW_FMT_STYLE:
 			fmt.style = name2type(get_c_string(value));
 			break;
 		default:
@@ -1001,7 +1071,7 @@ static LISP lencode_format(LISP f)
 		}
 		
 	}
-	result = encode_format(~0, &fmt);
+	result = MwEncodeFormat(~0, &fmt);
 	return flocons(result);
 }
 
@@ -1024,9 +1094,9 @@ associative list of the format
 */
 static LISP ldecode_format(LISP f)
 {
-	sfmt fmt;
+	MwFmt fmt;
 
-	decode_format(get_c_long(f), ~0, &fmt);
+	MwDecodeFormat(get_c_long(f), ~0, &fmt);
 	return cons(SS("family", fmt.family),
 		 cons(SL("size", fmt.size),
 		   cons(SS("fg", fmt.fg),
@@ -1059,13 +1129,13 @@ static void insert_plugin(int mode, char *plugin_name)
 {
         static char path[1024], name[1024];
         char fn[1024];
-        char pn[1024];
+        char p[1024], pn[1024];
         char cmd[1024];
         char fmt[80];
-        char *p;
         buffer *buf = w_list->buf;
         int n;
         plugin_t plugin;
+	int sht = w_list->sht;
 
         /* ask for file name */
         if (path[0] == '\0') getcwd(path, 1024);
@@ -1077,7 +1147,7 @@ static void insert_plugin(int mode, char *plugin_name)
                 else strcpy(name, plugin_name);
                 strcpy(fn, plugin_name);
         } else {
-	        if (!select_file(path, name, plugin_patterns, fmt)) return;
+	        if (!select_file(path, name, plugin_patterns, fmt, 0)) return;
         	sprintf(fn, "%s/%s", path, name);
 	}
         plugin_unique_name(name, pn);
@@ -1087,15 +1157,15 @@ static void insert_plugin(int mode, char *plugin_name)
         plugin.displayed = 0;
 
         if (mode == PLUGIN_COPY) {
-                plugin.name = cstrdup(pn);
+                plugin.name = MwStrdup(pn);
                 /* copy the file */
-                p = plugin_basedir(buf, NULL);
+                plugin_basedir(p, buf->name);
                 sprintf(pn, "%s/%s", p, plugin.name);
                 sprintf(cmd, "(mkdir %s;cp %s %s)2>/dev/null", p, fn, pn);
                 system(cmd);
         } else {
                 strcpy(pn, fn);
-                plugin.name = cstrdup(pn);
+                plugin.name = MwStrdup(pn);
         }
 
         /* start the plugin */
@@ -1103,10 +1173,10 @@ static void insert_plugin(int mode, char *plugin_name)
         if (plugin.ph == -1) return;
 
         /* prepare the buffer */
-        n = buf->nplugin++;
-        buf->plugin = (plugin_t *)crealloc(buf->plugin,
-                buf->nplugin*sizeof(plugin_t));
-        buf->plugin[n] = plugin;
+        n = buf->sht[sht].nplugin++;
+        buf->sht[sht].plugin = (plugin_t *)MwRealloc(buf->sht[sht].plugin,
+                buf->sht[sht].nplugin*sizeof(plugin_t));
+        buf->sht[sht].plugin[n] = plugin;
         buf->change = TRUE;
         pr_scr_flag = TRUE;
 }
@@ -1137,11 +1207,12 @@ static int select_plugin(buffer *b)
 {
         char *plugin[20];
         int i;
+	int sht = w_list->sht;
 
-        if (b->nplugin == 0) return -1;
-        if (b->nplugin == 1) return 0;
-        for (i = 0; i < 20 && i < b->nplugin; i++)
-                plugin[i] = b->plugin[i].name;
+        if (b->sht[sht].nplugin == 0) return -1;
+        if (b->sht[sht].nplugin == 1) return 0;
+        for (i = 0; i < 20 && i < b->sht[sht].nplugin; i++)
+                plugin[i] = b->sht[sht].plugin[i].name;
         i = select_from_list("Select Plugin:", plugin, i);
         return i;
 }
@@ -1154,8 +1225,9 @@ static LISP lplugin_select(void)
 static LISP lplugin_export(void)
 {
         int n;
-        char fn[1024], pn[1024], cmd[1024], path[1024], fmt[80];
+        char p[1024], fn[1024], pn[1024], cmd[1024], path[1024], fmt[80];
         buffer *buf = w_list->buf;
+	int sht = w_list->sht;
 
         /* pick plugin from list */
         n = select_plugin(buf);
@@ -1164,11 +1236,12 @@ static LISP lplugin_export(void)
         /* get description of the handler */
         /* ask for file name */
         getcwd(path, 1024);
-        strcpy(fn, buf->plugin[n].name);
-        if (!select_file(path, fn, NULL, fmt)) return NIL;
+        strcpy(fn, buf->sht[sht].plugin[n].name);
+        if (!select_file(path, fn, NULL, fmt, 1)) return NIL;
 
         /* save the file */
-        sprintf(pn, "%s/%s", plugin_basedir(buf, NULL), buf->plugin[n].name);
+        sprintf(pn, "%s/%s", plugin_basedir(p, buf->name),
+			buf->sht[sht].plugin[n].name);
         sprintf(cmd, "cp %s %s/%s", pn, path, fn);
         system(cmd);
         return NIL;
@@ -1178,17 +1251,18 @@ static LISP lplugin_delete(void)
 {
         int n;
         buffer *buf = w_list->buf;
+	int sht = w_list->sht;
 
         /* pick plugin from list */
         n = select_plugin(buf);
         if (n == -1) return NIL;
 
         /* remove plugin */
-        plugin_stop(buf->plugin[n].ph);
-        cfree(buf->plugin[n].name);
-        buf->nplugin--;
-        for (; n < buf->nplugin; n++)
-                buf->plugin[n] = buf->plugin[n+1];
+        plugin_stop(buf->sht[sht].plugin[n].ph);
+        MwFree(buf->sht[sht].plugin[n].name);
+        buf->sht[sht].nplugin--;
+        for (; n < buf->sht[sht].nplugin; n++)
+                buf->sht[sht].plugin[n] = buf->sht[sht].plugin[n+1];
         buf->change = pr_scr_flag = TRUE;
         return NIL;
 }
@@ -1197,17 +1271,225 @@ static LISP lplugin_move(void)
 {
         int n;
         buffer *buf = w_list->buf;
+	int sht = w_list->sht;
 
         /* pick plugin from list */
         n = select_plugin(buf);
         if (n == -1) return NIL;
 
-        buf->plugin[n].row = 1;
-        buf->plugin[n].col = 1;
+        buf->sht[sht].plugin[n].row = 1;
+        buf->sht[sht].plugin[n].col = 1;
         buf->change = pr_scr_flag = TRUE;
         return NIL;
 }
 
+static LISP lpaper_get(LISP b)
+{
+	char *n;
+	int w, h, o;
+	if (NULLP(b)) {
+		n = paper_name;
+		w = paper_width;
+		h = paper_height;
+		o = orientation;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		n = buf->paper_name;
+		w = buf->paper_width;
+		h = buf->paper_height;
+		o = buf->orientation;
+	}
+	return cons(strcons(strlen(n), n),
+	         cons(flocons(w),
+		   cons(flocons(h),
+		     cons(flocons(o), NIL))));
+}
+
+static LISP lpaper_set(LISP b, LISP p)
+{
+	char *n;
+	int w, h, o;
+
+	n = MwStrdup(get_c_string(CAR(p)));
+	w = get_c_long(CAR(CDR(p)));
+	h = get_c_long(CAR(CDR(CDR(p))));
+	o = get_c_long(CAR(CDR(CDR(CDR(p)))));
+	if (NULLP(b)) {
+		paper_name = n;
+		paper_width = w;
+		paper_height = h;
+		orientation = o;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		buf->paper_name = n;
+		buf->paper_width = w;
+		buf->paper_height = h;
+		buf->orientation = o;
+		buf->change = pr_scr_flag = TRUE;
+	}
+	return NIL;
+}
+
+static LISP lmargin_get(LISP b)
+{
+	int tm, bm, lm, rm, hm, fm;
+
+	if (NULLP(b)) {
+		tm = top_margin;
+		bm = bottom_margin;
+		lm = left_margin;
+		rm = right_margin;
+		hm = header_margin;
+		fm = footer_margin;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		tm = buf->top_margin;
+		bm = buf->bottom_margin;
+		lm = buf->left_margin;
+		rm = buf->right_margin;
+		hm = buf->header_margin;
+		fm = buf->footer_margin;
+	}
+	return cons(flocons(tm),
+		 cons(flocons(bm),
+		   cons(flocons(lm),
+		     cons(flocons(rm),
+		       cons(flocons(hm),
+			 cons(flocons(fm), NIL))))));
+}
+
+static LISP lmargin_set(LISP b, LISP m)
+{
+	int tm, bm, lm, rm, hm, fm;
+
+	tm = get_c_long(CAR(m));
+	bm = get_c_long(CAR(CDR(m)));
+	lm = get_c_long(CAR(CDR(CDR(m))));
+	rm = get_c_long(CAR(CDR(CDR(CDR(m)))));
+	hm = get_c_long(CAR(CDR(CDR(CDR(CDR(m))))));
+	fm = get_c_long(CAR(CDR(CDR(CDR(CDR(CDR(m)))))));
+	if (NULLP(b)) {
+		top_margin = tm;
+		bottom_margin = bm;
+		left_margin = lm;
+		right_margin = rm;
+		header_margin = hm;
+		footer_margin = fm;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		buf->top_margin = tm;
+		buf->bottom_margin = bm;
+		buf->left_margin = lm;
+		buf->right_margin = rm;
+		buf->header_margin = hm;
+		buf->footer_margin = fm;
+		buf->change = pr_scr_flag = TRUE;
+	}
+	return NIL;
+}
+
+static LISP lheadfoot_get(LISP b)
+{
+	char *h, *f;
+	int fp;
+
+	if (NULLP(b)) {
+		h = header;
+		f = footer;
+		fp = header_on_first;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		h = buf->header;
+		f = buf->footer;
+		fp = buf->header_on_first;
+	}
+	return cons(strcons(strlen(h), h),
+		 cons(strcons(strlen(f), f),
+		   cons(flocons(fp), NIL)));
+}
+
+static LISP lheadfoot_set(LISP b, LISP hf)
+{
+	char *h, *f;
+	int fp;
+
+	h = MwStrdup(get_c_string(CAR(hf)));
+	f = MwStrdup(get_c_string(CAR(CDR(hf))));
+	fp = get_c_long(CAR(CDR(CDR(hf))));
+	if (NULLP(b)) {
+		header = h;
+		footer = f;
+		header_on_first = fp;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		buf->header = h;
+		buf->footer = f;
+		buf->header_on_first = fp;
+		buf->change = pr_scr_flag = TRUE;
+	}
+	return NIL;
+}
+
+static LISP lfirst_page_get(LISP b)
+{
+	int n;
+
+	if (NULLP(b)) {
+		n = first_page_number;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		n = buf->first_page_number;
+	}
+	return flocons(n);
+}
+
+static LISP lfirst_page_set(LISP b, LISP no)
+{
+	int n = get_c_long(no);
+
+	if (NULLP(b)) {
+		first_page_number = n;
+	} else {
+		buffer *buf;
+		char *bn = get_c_string(b);
+		if (bn[0]) buf = find_buffer_by_name(bn);
+		else buf = w_list->buf;
+		if (!buf) return NIL;
+		buf->first_page_number = n;
+		buf->change = pr_scr_flag = TRUE;
+	}
+	return NIL;
+}
+
+
 static LISP lmake_backups(LISP p)
 {
 	make_backups = get_c_long(p);
@@ -1215,7 +1497,7 @@ static LISP lmake_backups(LISP p)
 }
 
 /* Commands that take no arguments */
-s_fn_table fn_table[] =
+s_fn_table0 fn_table[] =
 {
 	{"listsel-test", listsel_test},
 
@@ -1224,9 +1506,7 @@ s_fn_table fn_table[] =
 
 	/* editing */
 
-	/* block commands */
-	{"preview", preview_block},
-	{"print-block", print_block},
+	{"psformat", lpsformat},
 
 	/* new window */
 	{"delete-window", delete_window},
@@ -1249,7 +1529,12 @@ s_fn_table fn_table[] =
 
 	{"quit-program", quit_egon},
 	/* low level functions */
-	{"bell", bell},
+
+	{"add-sheet", ladd_sheet},
+	{"remove-sheet", lremove_sheet},
+	{"rename-sheet", lrename_sheet},
+	{"get-sheet", lget_sheet},
+	{"get-bgrad", get_bgrad},
 
 	{"ani-begin", ani_begin},
 	{"ani-end", ani_end},
@@ -1259,14 +1544,16 @@ s_fn_table fn_table[] =
 	{"get-format", get_format},
 	{"save-preferences", save_preferences},
 	{"abort", labort},
-	{(char *) 0, (LISP(*)())0}
+	{"aboutbox", laboutbox},
+	{"aboutsiag", laboutsiag},
+	{(char *) 0, (LISP(*)(void))0}
 };
 
 /* Commands that take 1 argument */
 
-s_fn_table fn_table1[] = {
+s_fn_table1 fn_table1[] = {
 	/* low level functions */
-	{"spawn", spawn},
+	{"spawn", lspawn},
 	{"execute-interpreter-command", execute_interpreter_command},
 	{"buffer-changed", buffer_changed},
 	{"execute", lexecute},
@@ -1274,6 +1561,7 @@ s_fn_table fn_table1[] = {
 
 	{"ani-object", lani_object},
 	{"ani-background", ani_background},
+	{"ani-bgrad", ani_bgrad},
 	{"ani-time", ani_time},
 	{"ani-duration", lani_duration},
 	{"ani-delta", lani_delta},
@@ -1283,17 +1571,25 @@ s_fn_table fn_table1[] = {
 	{"set-format", set_format},
 	{"decode-format", ldecode_format},
 	{"encode-format", lencode_format},
+	{"paper-get", lpaper_get},
+	{"margin-get", lmargin_get},
+	{"headfoot-get", lheadfoot_get},
+	{"first-page-get", lfirst_page_get},
 	{"make-backups", lmake_backups},
 	{NULL, NULL}
 };
 
 /* Commands that take 2 arguments */
-s_fn_table fn_table2[] = {
+s_fn_table2 fn_table2[] = {
 	{"ask-for-str", lask_for_str},
 	{"alertbox", lalertbox},
 
 	{"ani-geometry", lani_geometry},
-	{"font-alias", lfont_alias},
+	{"move-sheet", lmove_sheet},
+	{"paper-set", lpaper_set},
+	{"margin-set", lmargin_set},
+	{"headfoot-set", lheadfoot_set},
+	{"first-page-set", lfirst_page_set},
 	{NULL, NULL}
 };
 
@@ -1311,10 +1607,7 @@ void init_cmds(void)
 		init_subr_1(fn_table1[i].name, fn_table1[i].function);
 	for (i = 0; fn_table2[i].name; i++)
 		init_subr_2(fn_table2[i].name, fn_table2[i].function);
-	init_subr_5("register-x-font", lregister_x_font);
-	init_subr_5("register-ps-font", lregister_ps_font);
-	init_subr_5("register-t1-font", lregister_t1_font);
-	init_subr_4("register-color", lregister_color);
+	init_subr_3("savematrix", lsavematrix);
 	init_subr_3("plugin-register", lplugin_register);
 	init_subr_0("plugin-select", lplugin_select);
         init_subr_1("plugin-import", lplugin_import);
