@@ -1,6 +1,6 @@
 /*
    Egon Animator
-   Copyright (C) 1997-1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,8 +38,8 @@ fileio_c.c
 #include <ctype.h>
 
 #include "egon.h"
-#include "types.h"
 #include "../common/common.h"
+#include <Mowitz/MwUtils.h>
 
 /* ---
 Returns: 0 if successful, otherwise 1
@@ -47,8 +47,8 @@ Returns: 0 if successful, otherwise 1
 
 static int save(char *fn, buffer * buf)
 {
-	ani_object *o;
-	ani_script *s;
+	MwAniObject *o;
+	MwAniScript *s;
 	FILE *fp = fopen(fn, "w");
 	char basename[256], *p;
 	int i, j;
@@ -64,20 +64,20 @@ static int save(char *fn, buffer * buf)
 	for (; *p; p++)
 		if (!isalnum(*p)) *p = '_';
 
-	fprintf(fp, "int %s_delta = %d;\n", basename, buf->delta);
-	fprintf(fp, "int %s_duration = %d;\n", basename, buf->duration);
+	fprintf(fp, "int %s_delta = %d;\n", basename, buf->sht[0].delta);
+	fprintf(fp, "int %s_duration = %d;\n", basename, buf->sht[0].duration);
 	fprintf(fp, "int %s_width = %d;\n", basename, buf->width);
 	fprintf(fp, "int %s_height = %d;\n", basename, buf->height);
-	if (buf->bg)
-		fprintf(fp, "char *%s_background = \"%s\";\n", basename, buf->bg);
+	if (buf->sht[0].bg)
+		fprintf(fp, "char *%s_background = \"%s\";\n", basename, buf->sht[0].bg);
 	else
 		fprintf(fp, "char *%s_background = NULL;\n", basename);
 
 	/* save the scripts for all the objects */
 	i = 0;
-	for (o = buf->cast; o; o = o->next) {
+	for (o = buf->sht[0].cast; o; o = o->next) {
 		j = 0;
-		fprintf(fp, "ani_script %s_script_%d[] = {\n", basename, i);
+		fprintf(fp, "MwAniScript %s_script_%d[] = {\n", basename, i);
 		for (s = o->script; s; s = s->next)  {
 			if (s->next) {	/* not the last one */
 				fprintf(fp, "\t{%d, %d, %d, %d, %d, %d, &%s_script_%d[%d]},\n",
@@ -95,9 +95,9 @@ static int save(char *fn, buffer * buf)
 	}
 
 	/* then save the objects */
-	fprintf(fp, "ani_object %s_objects[] = {\n", basename);
+	fprintf(fp, "MwAniObject %s_objects[] = {\n", basename);
 	i = 0;
-	for (o = buf->cast; o; o = o->next) {
+	for (o = buf->sht[0].cast; o; o = o->next) {
 		fprintf(fp, "/* Object %s */\n", o->name);
 		if (o->next) {	/* not the last one */
 			fprintf(fp, "\t{%d, \"%s\", %s_script_%d, %d, \"%s\", &%s_objects[%d]},\n",
@@ -132,7 +132,7 @@ static int myformat(char *fn)
 	int result;
 
 	result = ((ext = strrchr(fn, '.')) &&
-		!cstrcasecmp(ext, ".c") &&
+		!MwStrcasecmp(ext, ".c") &&
 		(fp = fopen(fn, "r")) &&
 		fgets(b, sizeof b, fp) &&
 		!strncmp(b, C_MAGIC, strlen(C_MAGIC)));
@@ -144,7 +144,7 @@ static int myformat(char *fn)
 Register this file format.
 */
 
-void fileio_c_init()
+void fileio_c_init(void)
 {
 	register_format(NULL, save, myformat, "C source (*.c)");
 }
