@@ -1,6 +1,6 @@
 /*
    Egon Animator
-   Copyright (C) 1997-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,12 +52,10 @@ fileio_egon.c
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "../common/oldfonts.h"
 #include "egon.h"
-#include "types.h"
 #include "user_interface.h"
 #include "../common/common.h"
-#include "../common/cmalloc.h"
+#include <Mowitz/MwUtils.h>
 #include "../common/plugin.h"
 
 int make_backups = 1;
@@ -69,10 +67,10 @@ returns the name of a component in a structured file. Caller must free
 static char *file_name(buffer *buf, char *n)
 {
         char b[1024];
-        plugin_basedir(buf, b);
+        plugin_basedir(b, buf->name);
         strcat(b, "/");
         strcat(b, n);
-        return cstrdup(b);
+        return MwStrdup(b);
 }
 
 /* ---
@@ -84,48 +82,52 @@ Returns: 0 if successful, otherwise 1
 
 static int save_flat(char *fn, buffer * buf)
 {
-	ani_object *o;
-	ani_script *s;
+	MwAniObject *o;
+	MwAniScript *s;
 	char *string;
 	FILE *fp = fopen(fn, "w");
 	char b[1024];
-	char *used_fmt = ccalloc(nformat+1, 1);
+	char *used_fmt = MwCalloc(MwFormatCount+1, 1);
 	int fmt;
+	int sht;
 
 	if (!fp) return 1;
 
-	plugin_basedir(buf, b);
+	plugin_basedir(b, buf->name);
 
 	fprintf(fp, "# Creator: %s\n", version);
 	fprintf(fp, "# %s\n", buf->name);
-	fprintf(fp, ".delta %d\n", buf->delta);
-	fprintf(fp, ".duration %d\n", buf->duration);
 	fprintf(fp, ".geometry %d %d\n", buf->width, buf->height);
-	if (buf->bg)
-		fprintf(fp, ".background %s\n", buf->bg);
-	for (o = buf->cast; o; o = o->next) {
-		fprintf(fp, "# Object %s\n", o->name);
-		/* strip off directory part of image files in
-		   a structured file */
-		if (o->type == ANI_PIXMAP &&
-				!strncmp(o->string, b, strlen(b)))
-			string = strrchr(o->string, '/')+1;
-		else
-			string = o->string;
-		fmt = o->fmt;
-		if (!used_fmt[fmt]) {
-			save_formats(fp, fmt);
-			used_fmt[fmt] = 1;
+	for (sht = 0; sht < buf->nsht; sht++) {
+		fprintf(fp, ".sheet %d %s\n", sht, buf->sht[sht].name);
+		fprintf(fp, ".delta %d\n", buf->sht[sht].delta);
+		fprintf(fp, ".duration %d\n", buf->sht[sht].duration);
+		if (buf->sht[sht].bg)
+			fprintf(fp, ".background %s\n", buf->sht[sht].bg);
+		for (o = buf->sht[sht].cast; o; o = o->next) {
+			fprintf(fp, "# Object %s\n", o->name);
+			/* strip off directory part of image files in
+			   a structured file */
+			if (o->type == MW_ANI_PIXMAP &&
+					!strncmp(o->string, b, strlen(b)))
+				string = strrchr(o->string, '/')+1;
+			else
+				string = o->string;
+			fmt = o->fmt;
+			if (!used_fmt[fmt]) {
+				MwSaveFormats(fp, fmt);
+				used_fmt[fmt] = 1;
+			}
+			fprintf(fp, ".object %d %s %d %s\n",
+				o->type, o->name, o->fmt, string?string:"");
+			for (s = o->script; s; s = s->next)
+				fprintf(fp, ".tick %d %d %d %d %d %d\n",
+					s->time, s->x, s->y,
+					s->width, s->height, s->visible);
 		}
-		fprintf(fp, ".object %d %s %d %s\n",
-			o->type, o->name, o->fmt, string?string:"");
-		for (s = o->script; s; s = s->next)
-			fprintf(fp, ".tick %d %d %d %d %d %d\n",
-				s->time, s->x, s->y,
-				s->width, s->height, s->visible);
 	}
 	fclose(fp);
-	cfree(used_fmt);
+	MwFree(used_fmt);
 	return 0;
 }				/* save */
 
@@ -139,55 +141,66 @@ static int load_flat(char *fn, buffer *buf)
 {
 	FILE *fp;
 	char instring[1024], name[256], string[1024];
-	ani_object *o = NULL;
-	ani_script *s = NULL;
+	MwAniObject *o = NULL;
+	MwAniScript *s = NULL;
 	char b[1024];
 	int *fmts = NULL;
 	int nfmt = -1;
 	int f;
+	int sht = 0;
 
 	if ((fp = fopen(fn, "r")) == NULL)
 		return 1;
 
-	plugin_basedir(buf, b);
+	plugin_basedir(b, buf->name);
 
-	buf->delta = 100;
-	buf->duration = 4000;
-	buf->now = 0;
+	buf->sht[sht].delta = 100;
+	buf->sht[sht].duration = 4000;
+	buf->sht[sht].now = 0;
 	buf->width = 600;
 	buf->height = 400;
-	buf->cast = NULL;
-	buf->state = ANI_STOP;
-	buf->bg = NULL;
+	buf->sht[sht].cast = NULL;
+	buf->state = MW_ANI_STOP;
+	buf->sht[sht].bg = NULL;
 	buf->change = FALSE;
-	buf->plugin = NULL;
-	buf->nplugin = 0;
+	buf->sht[sht].plugin = NULL;
+	buf->sht[sht].nplugin = 0;
 
 	while (fgets(instring, 1024, fp) != NULL) {
 		if (instring[0] == '#') {
 			continue;
+		} else if (!strncmp(instring, ".sheet ", 7)) {
+			char name[1000];
+			sscanf(instring, ".sheet %d %[^\n]", &sht, name);
+			if (sht >= buf->nsht) {
+				buffer_add_sheet(buf, sht);
+			}
+			buffer_rename_sheet(buf, sht, name);
 		} else if (!strncmp(instring, ".delta ", 7)) {
-			sscanf(instring, ".delta %d", &buf->delta);
+			sscanf(instring, ".delta %d", &buf->sht[sht].delta);
 		} else if (!strncmp(instring, ".duration ", 10)) {
-			sscanf(instring, ".duration %d", &buf->duration);
+			sscanf(instring, ".duration %d", &buf->sht[sht].duration);
 		} else if (!strncmp(instring, ".geometry ", 10)) {
 			sscanf(instring, ".geometry %d %d",
 			&buf->width, &buf->height);
+		} else if (!strncmp(instring, ".background ", 12)) {
+			sscanf(instring, ".background %[^\n]", b);
+			buf->sht[sht].bg = MwStrdup(b);
 		} else if (!strncmp(instring, ".ft ", 4)) {
 			int n;
 			sscanf(instring, ".ft%d", &n);
 			if (n > nfmt) {
 				nfmt = n;
-				fmts = crealloc(fmts,
+				fmts = MwRealloc(fmts,
 						(nfmt+1)*sizeof *fmts);
 			}
-			fmts[n] = load_formats(fp);
+			fmts[n] = MwLoadFormats(fp);
 		} else if (!strncmp(instring, ".object ", 8)) {
-			if (buf->cast == NULL) {
-				o = (ani_object *)cmalloc(sizeof(ani_object));
-				buf->cast = o;
+			if (buf->sht[sht].cast == NULL) {
+				o = (MwAniObject *)MwMalloc(sizeof(MwAniObject));
+				buf->sht[sht].cast = o;
 			} else {
-				o->next = (ani_object *)cmalloc(sizeof(ani_object));
+				o->next = (MwAniObject *)MwMalloc(sizeof(MwAniObject));
 				o = o->next;
 			}
 			o->next = NULL;
@@ -195,30 +208,44 @@ static int load_flat(char *fn, buffer *buf)
 			name[0] = string[0] = '\0';
 			if (nfmt == -1) {	/* old */
 				int color, font;
-				sscanf(instring,
+				int n = sscanf(instring,
 					".object %d %s %d %d %[^\n]",
 					&o->type, name, &color, &font, string);
-					f = fmt_old2new(color<<COLOR_SHIFT|font);
+				if (n == 4) {
+					string[0] = '\0';
+					n++;
+				}
+				if (n != 5) {
+					fprintf(stderr, "Wrong arg count\n");
+					fclose(fp);
+					return 1;
+				}
+				f = MwFmtOldToNew(color<<MW_COLOR_SHIFT|font);
 			} else {
-				sscanf(instring,
+				int n = sscanf(instring,
 					".object %d %s %d %[^\n]",
 					&o->type, name, &f, string);
-					f = fmts[f];
+				if (n != 4) {
+					fprintf(stderr, "Wrong arg count\n");
+					fclose(fp);
+					return 1;
+				}
+				f = fmts[f];
 			}
 			o->fmt = f;
-			o->name = cstrdup(name);
-			if (o->type == ANI_PIXMAP && string[0] != '/') {
+			o->name = MwStrdup(name);
+			if (o->type == MW_ANI_PIXMAP && string[0] != '/') {
 				sprintf(instring, "%s/%s", b, string);
-				o->string = cstrdup(instring);
+				o->string = MwStrdup(instring);
 			} else {
-				o->string = cstrdup(string);
+				o->string = MwStrdup(string);
 			}
 		} else if (!strncmp(instring, ".tick ", 6)) {
 			if (o->script == NULL) {
-				s = (ani_script *)cmalloc(sizeof(ani_script));
+				s = (MwAniScript *)MwMalloc(sizeof(MwAniScript));
 				o->script = s;
 			} else {
-				s->next = (ani_script *)cmalloc(sizeof(ani_script));
+				s->next = (MwAniScript *)MwMalloc(sizeof(MwAniScript));
 				s = s->next;
 			}
 			s->next = NULL;
@@ -259,7 +286,9 @@ static int save(char *fn, buffer *buf)
         char cmd[1024], bak[1024];
         char *dir, *b, *p;
         int i, result;
-	ani_object *o;
+	MwAniObject *o = NULL;
+	int sht;
+	int structured = 0;
 
 	if (make_backups) {
 		/* make backup copy */
@@ -267,10 +296,20 @@ static int save(char *fn, buffer *buf)
 		rename(fn, bak);
 	}
 
-	for (o = buf->cast; o; o = o->next)
-		if (o->type == ANI_PIXMAP) break;
+	for (sht = 0; sht < buf->nsht; sht++) {
+		for (o = buf->sht[sht].cast; o; o = o->next) {
+			if (o->type == MW_ANI_PIXMAP) {
+				structured = 1;
+				break;
+			}
+		}
+		if (buf->sht[sht].nplugin) {
+			structured = 1;
+			break;
+		}
+	}
 
-        if (o == NULL && buf->nplugin == 0) {
+        if (!structured) {
 	        /* no images or plugins, save as usual */
                 return save_flat(fn, buf);
         }
@@ -280,19 +319,21 @@ static int save(char *fn, buffer *buf)
         p = strrchr(dir, '/');
         if (p) *p = '\0';
         mkdir(dir, 0700);
-        for (i = 0; i < buf->nplugin; i++) {
-                b = file_name(buf, buf->plugin[i].name);
-                plugin_save(buf->plugin[i].ph, b);
-                cfree(b);
-        }
+	for (sht = 0; sht < buf->nsht; sht++) {
+	        for (i = 0; i < buf->sht[sht].nplugin; i++) {
+	                b = file_name(buf, buf->sht[sht].plugin[i].name);
+	                plugin_save(buf->sht[sht].plugin[i].ph, b);
+	                MwFree(b);
+	        }
+	}
         /* save the main file */
         b = file_name(buf, "INDEX.egon");
         result = save_flat(b, buf);
-        cfree(b);
+        MwFree(b);
         /* tar up the lot */
         sprintf(cmd, "(cd %s;tar cf - *)>%s", dir, fn);
         system(cmd);
-        cfree(dir);
+        MwFree(dir);
         return result;
 }
 
@@ -303,14 +344,14 @@ So first try flat, then tar, then flat again.
 
 static int load(char *fn, buffer *buf)
 {
-        char cmd[1024];
+        char cmd[1024], b[1024];
         char *p;
         int n;
 
         if (flatfile(fn) || !tryuntar(fn, "INDEX.egon"))
                 return load_flat(fn, buf);
 
-        p = plugin_basedir(buf, NULL);
+        p = plugin_basedir(b, buf->name);
         sprintf(cmd,
                 "mkdir -p %s;"
                 "cat %s|(cd %s;tar xf -)",
@@ -318,7 +359,7 @@ static int load(char *fn, buffer *buf)
         system(cmd);
         p = file_name(buf, "INDEX.egon");
         n = load_flat(p, buf);
-        cfree(p);
+        MwFree(p);
         return n;
 }
 
@@ -330,7 +371,7 @@ static int myformat(char *fn)
 	int result;
 
 	result = ((ext = strrchr(fn, '.')) &&
-		!cstrcasecmp(ext, ".egon") &&
+		!MwStrcasecmp(ext, ".egon") &&
 		(fp = fopen(fn, "r")) &&
 		fgets(b, sizeof b, fp) &&
 		!strncmp(b, EGON_MAGIC, strlen(EGON_MAGIC)));
