@@ -39,13 +39,12 @@
 #include <X11/Xaw/Command.h>
 
 #include "../common/common.h"
-#include "../common/cmalloc.h"
-#include "../xcommon/filesel.h"
-#include "../xcommon/dialogs.h"
+#include <Mowitz/MwUtils.h>
 #include "../xcommon/xcommon.h"
 
 #include "gv.h"
 #include "../common/ps.h"
+#include <Mowitz/Mowitz.h>
 
 /* from dialogs.c */
 static String okay_accelerators =
@@ -105,7 +104,7 @@ CreateDialog(parent, name, okay_callback, cancel_callback)
 	     XtParseAcceleratorTable(okay_accelerators));	num_args++;
     okay = XtCreateManagedWidget("okay", commandWidgetClass,
 				 form, args, num_args);
-    label_set(okay, "OK");
+    MwLabelSet(okay, "OK");
     XtAddCallback(okay, XtNcallback, okay_callback, form);
 
 							num_args = 0;
@@ -118,7 +117,7 @@ CreateDialog(parent, name, okay_callback, cancel_callback)
     XtSetArg(args[num_args], XtNresizable, True);	num_args++;
     cancel = XtCreateManagedWidget("cancel", commandWidgetClass,
 				   form, args, num_args);
-    label_set(cancel, "Cancel");
+    MwLabelSet(cancel, "Cancel");
     XtAddCallback(cancel, XtNcallback, cancel_callback, parent);
 
     XtInstallAccelerators(response, okay);
@@ -137,7 +136,7 @@ SetDialogPrompt(w, newprompt)
     Widget label;
 
     label = XtNameToWidget(w, "prompt");
-    XtSetArg(args[0], XtNlabel, translate(newprompt));
+    XtSetArg(args[0], XtNlabel, _(newprompt));
     XtSetValues(label, args, ONE);
 }
 
@@ -248,7 +247,7 @@ fprintf(stderr, "Open file here. Is this code strange or what.\n");
     }
 }
 
-static char *select_file(void)
+static char *select_file(int e)
 {
 	static char path[1024], name[1024];
 	static char *patterns[] = {
@@ -269,11 +268,11 @@ static char *select_file(void)
 		need_init = 0;
 		name[0] = '\0';
 	}
-	n = fsel_input(toplevel, path, name, patterns, fmt, extra);
+	n = MwFileselInput(toplevel, path, name, patterns, fmt, extra, e);
 
 	if (n) {
 		sprintf(fn, "%s/%s", path, name);
-		return cstrdup(fn);
+		return MwStrdup(fn);
 	}
 	return NULL;
 }
@@ -283,7 +282,7 @@ void open_cb(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	FILE *fd;
 	struct stat st;
-	char *fn = select_file();
+	char *fn = select_file(0);
 	if (fn && ((fd = fopen(fn, "r")))) {
 		if (oldfilename) XtFree(oldfilename);
 		oldfilename = NULL;
@@ -292,7 +291,7 @@ void open_cb(Widget w, XtPointer client_data, XtPointer call_data)
 			XtFree(filename);
 		}
 		filename = XtNewString(fn);
-		cfree(fn);
+		MwFree(fn);
 		if (psfile) fclose(psfile);
 		psfile = fd;
 		stat(filename, &st);
@@ -305,16 +304,16 @@ void open_cb(Widget w, XtPointer client_data, XtPointer call_data)
 void save_cb(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	String error;
-	char *fn = select_file();
+	char *fn = select_file(1);
 	if (!fn) return;
 	if ((error = save_file(fn))) {
 	    char *buf = XtMalloc(strlen(error) + strlen(fn) + 2);
 	    sprintf(buf, "%s\n%s", error, fn);
-	    errorbox(toplevel, buf);
+	    MwErrorBox(toplevel, buf);
 	    XtFree(error);
 	    XtFree(buf);
 	}
-	cfree(fn);
+	MwFree(fn);
 }
 
 void printmarked_cb(Widget w, XtPointer client_data, XtPointer call_data)
@@ -353,11 +352,8 @@ static void do_help(char *p)
 {
 	char b[1024];
 
-	sprintf(b, "file:%s/%s", docdir, p);
-	if (!fork()) {
-		execlp(siaghelp, "Siaghelp", b, (char *)0);
-		_exit(0);
-	}
+	sprintf(b, "siagrun help file:%s/%s", docdir, p);
+	MwSpawn(b);
 }
 
 void help_cb(Widget w, XtPointer client, XtPointer call)
@@ -370,6 +366,20 @@ void copyright_cb(Widget w, XtPointer client, XtPointer call)
 	do_help("gvu/copyright.html");
 }
 
+void aboutgvu_cb(Widget w, XtPointer client, XtPointer call)
+{
+	char msg[1024];
+	sprintf(msg, "%s\n\n%s\n\nUlric Eriksson, ulric@siag.nu\n\n%s\n",
+		version,
+		_("A postscript viewer for X"),
+		_("Part of Siag Office"));
+	MwAboutBox(toplevel, "gvu.xpm", msg);
+}
+
+void aboutsiag_cb(Widget w, XtPointer client, XtPointer call)
+{
+	MwAboutSiag(toplevel);
+}
 
 /* Explicitly reopen the file. */
 void reopen_file(Widget w, XtPointer client_data, XtPointer call_data)
@@ -608,6 +618,34 @@ set_pagemedia(w, client_data, call_data)
     }
 }
 
+void
+set_antialiasing(w, client_data, call_data)
+    Widget w;
+    XtPointer client_data, call_data;
+{
+    Arg args[4];
+    int changed, old;
+
+    old = app_res.antialiasing;
+    app_res.antialiasing = 1 - (int) client_data;
+    changed = (app_res.antialiasing != old);
+
+    XtSetArg(args[0], XtNleft_icon, blank_bitmap);
+    XtSetValues(antialiasingentry[(int)app_res.antialiasing], args, ONE);
+    XtSetArg(args[0], XtNleft_icon, dot_bitmap);
+    XtSetValues(antialiasingentry[1-app_res.antialiasing], args, ONE);
+    if (changed) {
+	if(app_res.antialiasing)
+	  GhostviewSetAntialiasing(page); 
+        else
+	  GhostviewUnsetAntialiasing(page); 
+	GhostviewDisableInterpreter(page);
+	layout_ghostview();
+	show_page(current_page);
+    }
+}
+
+
 /* track mouse pointer and popup zoom window */
 void
 track_and_zoom(w, client_data, call_data)
