@@ -1,7 +1,7 @@
 /*
    clipart.c
 
-   Copyright (C) 1999  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1999-2001  Ulric Eriksson <ulric@siag.nu>
 
    Based on Tim Theisen's Ghostview.
 
@@ -54,31 +54,11 @@ extern int errno;
 #include <X11/Shell.h>
 #include <X11/Xatom.h>
 #include <X11/Xaw/Cardinals.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Scrollbar.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/IntrinsicP.h>
-#include <X11/Xaw/TextP.h>
-#include <X11/Xmu/StdCmap.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Command.h>
 
-#include "../common/cmalloc.h"
 #include "../common/ps.h"
 #include "../common/common.h"
 #include "../xcommon/xcommon.h"
-#include "../xcommon/Rudegrid.h"
-#include "../xcommon/Frame.h"
-#include "../xcommon/filesel.h"
-#include "../xcommon/dialogs.h"
-
-#define XtSetFloatArg(arg, n, d) \
-    if (sizeof(float) > sizeof(XtArgVal)) { \
-	XtSetArg(arg, n, &(d)); \
-    } else { \
-	XtArgVal *ld = (XtArgVal *)&(d); \
-	XtSetArg(arg, n, *ld); \
-    }
+#include <Mowitz/Mowitz.h>
 
 #include "../xcommon/Ghostview.h"
 
@@ -92,12 +72,6 @@ String fallback_resources[] = {
 "Clipart*topbox.yLayout:	100%",
 "Clipart*input:			True",
 "Clipart*allowShellResize:		True",
-"Clipart*SimpleMenu.translations:	#replace \\n\
-    <EnterWindow>:	highlight() \\n\
-    <LeaveWindow>:	unhighlight() \\n\
-    <BtnMotion>:	highlight() \\n\
-    <Btn2Up>:	MenuPopdown() GhostviewForce() notify() unhighlight() \\n\
-    <BtnUp>:	MenuPopdown() GhostviewDefault() notify() unhighlight()",
 "Clipart*Ghostview.background:	white",
 "Clipart.translations:		#replace \\n\
     <MapNotify>:		GhostviewCheckFile() \\n\
@@ -114,16 +88,12 @@ String fallback_resources[] = {
     <Btn2Down>:	notify(120 120 300 300) \\n\
     <Btn3Down>:	notify(90 90 400 400)",
 "Clipart*topbox.translations:	#replace \\n\
-    <Key>Q:	GhostviewQuit() \\n\
-    <Key>R:	GhostviewReopen() \\n\
-    <Key>period:	GhostviewShow() \\n\
+    <Key>Q:	GhostviewQuit() \\n"
+    "<Key>period:	GhostviewShow() \\n\
     Ctrl<Key>L:	GhostviewShow()",
     NULL
 };
 
-#define ROWS 22
-#define COLS 68
-
 #ifndef max
 #define max(a, b)	((a) > (b) ? (a) : (b))
 #endif
@@ -168,8 +138,6 @@ static FILE	*psfile;
 static String	filename;
 static String	oldfilename;
 static int	current_page;
-static int	current_magstep;
-static XtPageOrientation	current_orientation;
 static int	default_pagemedia;
 static int	current_pagemedia;
 static Boolean	force_document_media;
@@ -179,15 +147,13 @@ static int	current_lly;
 static int	current_urx;
 static int	current_ury;
 static int	base_papersize;
-static Boolean	info_up;
 static int	toc_length;
 static int	toc_entry_length;
-static int	info_length;
 static time_t	mtime;
 static struct document *doc;
 static struct document *olddoc;
 static Atom	wm_delete_window;
-static int	catch_Xerror();
+static int	catch_Xerror(Display *, XErrorEvent *);
 static XErrorHandler	old_Xerror;
 static Boolean	dying;
 static XErrorEvent	bomb;
@@ -199,66 +165,34 @@ static AppResources app_res;
 
 /* Widgets */
 static Widget toplevel;
-static Widget     topbox;
-static Widget             page;
-/* Popup widgets */
-static Widget infopopup;
-static Widget     infoform;
-static Widget         infotext;
-static Widget dialogpopup;
+static Widget topbox;
+static Widget page;
 
 static void add_actions(XtAppContext);
 
 /* Callbacks */
-static void quit_ghostview();
-static void reopen_file();
-static void this_page();
-static void message();
-static void output();
-static void destroy_ghost();
+static void quit_ghostview(Widget, XtPointer, XtPointer);
+static void this_page(Widget, XtPointer, XtPointer);
+static void message(Widget, XtPointer, XtPointer);
+static void destroy_ghost(Widget, XtPointer, XtPointer);
 
 /* Misc */
-static void show_page();
-static Boolean setup_ghostview();
-static void magnify();
-static Boolean set_new_magstep();
-static Boolean set_new_orientation();
-static Boolean set_new_pagemedia();
-static void build_pagemedia_menu();
-static void new_file();
+static void show_page(int);
+static Boolean setup_ghostview(void);
+static Boolean set_new_pagemedia(int);
+static void new_file(int);
 
 /* Start application folding up by Destroying the top level widget. */
 /* The application exits when the last interpreter is killed during */
 /* a destroy callback from ghostview widgets. */
-static void quit_ghostview(w, client_data, call_data)
-    Widget w;
-    XtPointer client_data, call_data;
+static void quit_ghostview(Widget w, XtPointer client_data, XtPointer call_data)
 {
     XtDestroyWidget(toplevel);
 }
 
-/* Explicitly reopen the file. */
-static void reopen_file(w, client_data, call_data)
-    Widget w;
-    XtPointer client_data, call_data;
-{
-    struct stat sbuf;
-    int number = current_page;
-
-    fclose(psfile);
-    psfile = fopen(filename, "r");
-    mtime = sbuf.st_mtime;
-    if (oldfilename) XtFree(oldfilename);
-    oldfilename = XtNewString(filename);
-    new_file(number);
-    show_page(number);
-}
-
 /* Get the selection, if no selection, get the insertion point. */
 /* Show this page.  */
-static void this_page(w, client_data, call_data)
-    Widget w;
-    XtPointer client_data, call_data;
+static void this_page(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	GhostviewDisableInterpreter(page);
 	show_page(0);
@@ -266,9 +200,7 @@ static void this_page(w, client_data, call_data)
 
 /* Process messages from ghostscript */
 /* Refresh occurs when window was resized unexpectedly */
-static void message(w, client_data, call_data)
-    Widget w;
-    XtPointer client_data, call_data;
+static void message(Widget w, XtPointer client_data, XtPointer call_data)
 {
     int i;
     char *error = "Unknown error";
@@ -277,13 +209,11 @@ static void message(w, client_data, call_data)
 	if ((Widget)client_data == page) {
 	    error = "Error: PostScript interpreter failed in main window.\n\n";
 	}
-	output(w, NULL, error);
     } else if (!strcmp((char *) call_data, "BadAlloc")) {
 	if ((Widget)client_data == page) {
 	    error = 
 	       "Warning: Could not allocate backing pixmap in main window.\n\n";
 	}
-	output(w, NULL, error);
     } else if (!strcmp((char *) call_data, "Refresh")) {
 	    GhostviewSendPS(w, psfile, doc->beginprolog,
 			    doc->lenprolog, False);
@@ -298,41 +228,11 @@ static void message(w, client_data, call_data)
     }
 }
 
-/* Take output from ghostscript and display it in the infotext popup window */
-static void output(w, client_data, call_data)
-    Widget w;
-    XtPointer client_data, call_data;
-{
-    Arg args[2];
-    XawTextBlock message_block;
-
-    message_block.firstPos = 0;
-    message_block.length = strlen(call_data);
-    message_block.ptr = call_data;
-    message_block.format = FMT8BIT;
-
-    XawTextDisableRedisplay(infotext);
-
-    XtSetArg(args[0], XtNeditType, XawtextAppend);
-    XtSetValues(infotext, args, ONE);
-    XawTextReplace(infotext, info_length, info_length, &message_block);
-    info_length = info_length + message_block.length;
-
-    XtSetArg(args[0], XtNeditType, XawtextRead);
-    XtSetArg(args[1], XtNinsertPosition, info_length);
-    XtSetValues(infotext, args, TWO);
-    XawTextEnableRedisplay(infotext);
-    if (!info_up) XtPopup(infopopup, XtGrabNone);
-    info_up = True;
-}
-
 /* destroy callback for Ghostview widgets. */
 /* The disable interpreter call ensures that ghostscript is killed. */
 /* One the count goes to 0, we are sure that all forked processes have */
 /* been killed and that we can safely exit. */
-static void destroy_ghost(w, client_data, call_data)
-    Widget w;
-    XtPointer client_data, call_data;
+static void destroy_ghost(Widget w, XtPointer client_data, XtPointer call_data)
 {
     GhostviewDisableInterpreter((Widget) client_data);
     num_ghosts--;
@@ -345,64 +245,31 @@ static void destroy_ghost(w, client_data, call_data)
 /* from actions.c */
 
 /* Call the quit callback to stop ghostview */
-static void gv_quit(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+static void gv_quit(Widget w, XEvent *event,
+		String *params, Cardinal *num_params)
 {
 	if (!app_res.plugin) quit_ghostview(w, NULL, NULL);
 }
 
-/* Popup the open file dialog box. */
-static void gv_reopen(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
-{
-    reopen_file(w, NULL, NULL);
-}
-
 /* Call the this_page callback */
-static void gv_show(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+static void gv_show(Widget w, XEvent *event,
+		String *params, Cardinal *num_params)
 {
     this_page(w, NULL, NULL);
 }
 
 /* Implement WM_DELETE_WINDOW protocol */
-static void gv_delete_window(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+static void gv_delete_window(Widget w, XEvent *event,
+		String *params, Cardinal *num_params)
 {
     if (event->type == ClientMessage &&
 	event->xclient.data.l[0] != wm_delete_window) return;
     XtDestroyWidget(w);
 }
 
-/* dismiss a popup window */
-static void gv_dismiss(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
-{
-    XtPopdown(w);
-    if (w == infopopup) info_up = False;
-}
-
 /* Check to see if file was updated */
-static void gv_check_file(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+static void gv_check_file(Widget w, XEvent *event, String *params,
+		Cardinal *num_params)
 {
     struct stat sbuf;
 
@@ -415,10 +282,8 @@ static void gv_check_file(w, event, params, num_params)
 
 static XtActionsRec actions[] = {
     {"GhostviewQuit",		gv_quit},
-    {"GhostviewReopen",		gv_reopen},
     {"GhostviewShow",		gv_show},
     {"GhostviewDeleteWindow",	gv_delete_window},
-    {"GhostviewDismiss",	gv_dismiss},
     {"GhostviewCheckFile",	gv_check_file},
 };
 
@@ -611,9 +476,7 @@ static struct {
  */
 	{"WIN", win},
 	{"QUIT", quit},
-#if 1	/* currently broken, doesn't handle scaling */
         {"PRNT", prnt},
-#endif
 	{ NULL, NULL }
 };
 
@@ -644,10 +507,6 @@ static void read_plugin_cmd(XtPointer client_data, int *fid, XtInputId * id)
 	fflush(stdout);
 }
 
-
-/* Popup children */
-static        Widget         infodismiss;
-
 int main(int argc, char **argv)
 {
     struct stat sbuf;
@@ -657,9 +516,6 @@ int main(int argc, char **argv)
     Cardinal num_args;
     int i;
     String s1;
-    XFontStruct *font;
-    Dimension bottomMargin, leftMargin, rightMargin, topMargin;
-    Dimension width, height;
 
     common_init("Clipart %s. No Warranty.");
 
@@ -678,12 +534,34 @@ int main(int argc, char **argv)
     }
     if (argc > 2) exit(EXIT_FAILURE);
     if (argc == 2) {
+	char *p, b[2000];
 	filename = XtNewString(argv[1]);
+	p = strrchr(filename, '.');
+	if (p && !MwStrcasecmp(p, ".tex")) {
+		sprintf(b,
+			"(cp %s %s/clipart.tex &&"
+			"cd %s &&"
+			"latex clipart.tex &&"
+			"dvips -E clipart.dvi -o clipart.ps) > /dev/null 2>&1",
+			filename, siag_tmpdir, siag_tmpdir);
+		system(b);
+		sprintf(b, "%s/clipart.ps", siag_tmpdir);
+		filename = XtNewString(b);
+	}
+	else if (p && !MwStrcasecmp(p, ".dvi")) {
+		sprintf(b,
+			"(cp %s %s/clipart.dvi &&"
+			"dvips -E clipart.dvi -o clipart.ps) > /dev/null 2>&1",
+			filename, siag_tmpdir);
+		system(b);
+		sprintf(b, "%s/clipart.ps", siag_tmpdir);
+		filename = XtNewString(b);
+	}
 	if (strcmp(filename, "-")) {
 #ifdef VMS
-	    if ((psfile = fopen(argv[1], "r", "mbc=100")) == NULL) {
+	    if ((psfile = fopen(filename, "r", "mbc=100")) == NULL) {
 #else
-	    if ((psfile = fopen(argv[1], "r")) == NULL) {
+	    if ((psfile = fopen(filename, "r")) == NULL) {
 #endif
 		printf("501 Cannot open\n");
 		fprintf(stderr, "Cannot open ");
@@ -707,60 +585,10 @@ int main(int argc, char **argv)
 			      resources, XtNumber(resources), NULL, ZERO);
     if ((s1 = getenv(app_res.printer_variable))) app_res.default_printer = s1;
 
-    /* Instantiate Popup children */
-    infopopup = XtCreatePopupShell("information", topLevelShellWidgetClass,
-				   toplevel, NULL, ZERO);
-
-    infoform = XtCreateManagedWidget("form", formWidgetClass,
-				     infopopup, NULL, ZERO);
-
-							num_args = 0;
-    XtSetArg(args[num_args], XtNtop, XtChainTop);	num_args++;
-    XtSetArg(args[num_args], XtNbottom, XtChainBottom);	num_args++;
-    XtSetArg(args[num_args], XtNleft, XtChainLeft);	num_args++;
-    XtSetArg(args[num_args], XtNright, XtChainRight);	num_args++;
-    XtSetArg(args[num_args], XtNscrollHorizontal, XawtextScrollWhenNeeded);
-							num_args++;
-    XtSetArg(args[num_args], XtNscrollVertical, XawtextScrollWhenNeeded);
-							num_args++;
-    XtSetArg(args[num_args], XtNdisplayCaret, False);	num_args++;
-    infotext = XtCreateManagedWidget("text", asciiTextWidgetClass,
-				     infoform, args, num_args);
-
-							num_args = 0;
-    XtSetArg(args[num_args], XtNfromVert, infotext);	num_args++;
-    XtSetArg(args[num_args], XtNtop, XtChainBottom);	num_args++;
-    XtSetArg(args[num_args], XtNbottom, XtChainBottom);	num_args++;
-    XtSetArg(args[num_args], XtNleft, XtChainLeft);	num_args++;
-    XtSetArg(args[num_args], XtNright, XtChainRight);	num_args++;
-    infodismiss = XtCreateManagedWidget("dismiss", commandWidgetClass,
-				       infoform, args, num_args);
-							num_args = 0;
-    XtSetArg(args[num_args], XtNfont, &font);		num_args++;
-    XtSetArg(args[num_args], XtNbottomMargin, &bottomMargin);num_args++;
-    XtSetArg(args[num_args], XtNleftMargin, &leftMargin);num_args++;
-    XtSetArg(args[num_args], XtNrightMargin, &rightMargin);num_args++;
-    XtSetArg(args[num_args], XtNtopMargin, &topMargin);	num_args++;
-    XtGetValues(infotext, args, num_args);
-
-    width = font->max_bounds.width * 80 + leftMargin + rightMargin;
-    height = (font->ascent + font->descent) * ROWS + topMargin + bottomMargin;
-
-    XtSetArg(args[0], XtNwidth, width);
-    XtSetArg(args[1], XtNheight, height);
-    XtSetValues(infotext, args, TWO);
-    XtSetValues(infodismiss, args, ONE);
-    XtRealizeWidget(infopopup);
-    XSetWMProtocols(dpy, XtWindow(infopopup), &wm_delete_window, 1);
-
-    dialogpopup = XtCreatePopupShell("popup", transientShellWidgetClass,
-				     toplevel, NULL, ZERO);
-
-
     /* Instantiate Widgets */
 
 	topbox = XtVaCreateManagedWidget("topbox",
-		rudegridWidgetClass, toplevel,
+		mwRudegridWidgetClass, toplevel,
 		(char *)0);
 
 	page = XtVaCreateManagedWidget("page",
@@ -769,7 +597,6 @@ int main(int argc, char **argv)
     num_ghosts++;
     XtAddCallback(page, XtNdestroyCallback, destroy_ghost, (XtPointer)page);
     XtAddCallback(page, XtNmessageCallback, message, (XtPointer)page);
-    XtAddCallback(page, XtNoutputCallback, output, (XtPointer)0);
 							num_args = 0;
     XtSetArg(args[num_args], XtNxdpi, &default_xdpi);	num_args++;
     XtSetArg(args[num_args], XtNydpi, &default_ydpi);	num_args++;
@@ -787,15 +614,11 @@ int main(int argc, char **argv)
 	app_res.magstep = app_res.minimum_magstep;
     if (app_res.magstep > app_res.maximum_magstep)
 	app_res.magstep = app_res.maximum_magstep;
-    set_new_magstep();
-    set_new_orientation(i);
     set_new_pagemedia(i);
 
     XtSetMappedWhenManaged(toplevel, False);
     XtRealizeWidget(toplevel);
 
-    XtSetArg(args[0], XtNtransientFor, toplevel);
-    XtSetValues(dialogpopup, args, ONE);
     XSetWMProtocols(dpy, XtWindow(toplevel), &wm_delete_window, 1);
 
     /* This sets the sizes on widget that were created during the realize. */
@@ -817,27 +640,7 @@ int main(int argc, char **argv)
     return 1;
 }
 
-
-/* Translate orientations defined by the enum in "ps.h" to
- * XtPageOrientations defined in "Ghostview.h".
- */
-static XtPageOrientation
-xorient(psorient)
-    int psorient;
-{
-    switch (psorient) {
-    case PORTRAIT: return XtPageOrientationPortrait;
-    case LANDSCAPE:
-	if (app_res.swap_landscape) {
-	    return XtPageOrientationSeascape;
-	} else {
-	    return XtPageOrientationLandscape;
-	}
-    }
-    return XtPageOrientationPortrait;
-}
-
-static void reset_size_hints()
+static void reset_size_hints(void)
 {
     Arg args[4];
     if (app_res.ncdwm) return;
@@ -848,17 +651,31 @@ static void reset_size_hints()
     XtSetValues(toplevel, args, FOUR);
 }
 
-static Boolean horiz_scroll_saved = False;
-static Boolean vert_scroll_saved = False;
+#define BOUNDINGBOX "%%BoundingBox:"
 
-static void reset_scroll_bars()
+static void read_page_size(int *x1, int *y1, int *x2, int *y2)
 {
-    if (horiz_scroll_saved || vert_scroll_saved) return;
+	char b[100];
+	FILE *fp = fopen(filename, "r");
+	*x1 = *y1 = *x2 = *y2 = 0;
+	if (fp == NULL) {
+		fprintf(stderr, "Can't open %s\n", filename);
+		return;
+	}
+	while (fgets(b, sizeof b, fp)) {
+		if (!MwStrncasecmp(b, BOUNDINGBOX, sizeof BOUNDINGBOX - 1)) {
+			sscanf(b+sizeof BOUNDINGBOX, "%d %d %d %d",
+				x1, y1, x2, y2);
+			break;
+		}
+	}
+	fclose(fp);
+	if (x1 >= x2) x2 = x1+100;
+	if (y1 >= y2) y2 = y1+100;
 }
 
 /* Start rendering a new page */
-static void show_page(number)
-    int number;
+static void show_page(int number)
 {
     struct stat sbuf;
     int i;
@@ -882,7 +699,7 @@ static void show_page(number)
     /* Coerce page number to fall in range */
 	number = 0;
 
-    if (set_new_orientation(number) || set_new_pagemedia(number))
+    if (/*set_new_orientation(number) ||*/ set_new_pagemedia(number))
 		;
     if (1) {
 	current_page = number;
@@ -904,7 +721,28 @@ return;
 	else
 	    i = current_page;
 if (doc && (doc->pages == NULL)) {
-fprintf(stderr, "Loaded doc but no page information\n");
+read_page_size(&doc->boundingbox[LLX],
+		&doc->boundingbox[LLY],
+		&doc->boundingbox[URX],
+		&doc->boundingbox[URY]);
+reset_size_hints();
+current_llx = doc->boundingbox[LLX];
+current_lly = doc->boundingbox[LLY];
+current_urx = doc->boundingbox[URX];
+current_ury = doc->boundingbox[URY];
+if (current_llx < current_urx && current_lly < current_ury) {
+XtVaSetValues(page,
+	XtNllx, current_llx,
+	XtNlly, current_lly,
+	XtNurx, current_urx,
+	XtNury, current_ury,
+	(char *)0);
+XtVaSetValues(toplevel,
+	XtNwidth, current_urx-current_llx,
+	XtNheight, current_ury-current_lly,
+	(char *)0);
+}
+set_new_pagemedia(0);
 return;
 }
 
@@ -935,7 +773,6 @@ Boolean setup_ghostview(void)
     Arg args[20];
     Cardinal num_args;
     int oldtoc_entry_length = 0;
-    XawTextBlock message_block;
 
     /* Reset to a known state. */
     psfree(olddoc);
@@ -946,8 +783,6 @@ Boolean setup_ghostview(void)
     /* Scan document and start setting things up */
     if (psfile) doc = psscan(psfile);
 
-    build_pagemedia_menu();
-
     /* Reset ghostscript and output messages popup */
     if (!doc || !olddoc ||
 	strcmp(oldfilename, filename) ||
@@ -957,16 +792,6 @@ Boolean setup_ghostview(void)
 	olddoc->endsetup != doc->endsetup) {
 
 	GhostviewDisableInterpreter(page);
-	XtPopdown(infopopup);
-	info_up = False;
-	XtSetArg(args[0], XtNeditType, XawtextEdit);
-	XtSetArg(args[1], XtNinsertPosition, 0);
-	XtSetValues(infotext, args, TWO);
-	message_block.length = 0;
-	XawTextReplace(infotext, 0, info_length, &message_block);
-	info_length = 0;
-	XtSetArg(args[0], XtNeditType, XawtextRead);
-	XtSetValues(infotext, args, ONE);
     }
 
     /* Build table of contents */
@@ -1032,10 +857,7 @@ find_page(label)
  */
 static unsigned int delay = 125;	/* Start with 1/8 second delay */
 
-static void
-try_try_again(client_data, timer)
-    XtPointer client_data;
-    XtIntervalId *timer;
+static void try_try_again(XtPointer client_data, XtIntervalId *timer)
 {
     XSync(XtDisplay(toplevel), False);	/* Push everything out */
     if (XtAppPending(app_con)) {
@@ -1048,18 +870,6 @@ try_try_again(client_data, timer)
     }
 }
 
-/* Compute new dpi from magstep */
-static void magnify(dpi, magstep)
-    float *dpi;
-    int    magstep;
-{
-    if (magstep < 0) {
-	while (magstep++) *dpi /= 1.2;
-    } else {
-	while (magstep--) *dpi *= 1.2;
-    }
-}
-
 /* Attempt to open file, return error message string on failure */
 String open_file(String name)
 {
@@ -1112,11 +922,7 @@ String save_file(String name)
 	String buf = XtMalloc(strlen(app_res.save_fail) +
 			      strlen(se/*sys_errlist[errno]*/) + 1);
 	strcpy(buf, app_res.save_fail);
-#if 0
-	if (errno <= sys_nerr) strcat(buf, sys_errlist[errno]);
-#else
 	strcat(buf, se);
-#endif
 	return buf;
     } else {
 	fclose(pswrite);
@@ -1124,74 +930,6 @@ String save_file(String name)
     }
 }
 
-/* Set new magstep */
-Boolean set_new_magstep(void)
-{
-    int new_magstep;
-    Boolean changed = False;
-    Arg args[20];
-    Cardinal num_args;
-    float xdpi, ydpi;
-
-    new_magstep = app_res.magstep;
-    /* If magstep changed, stop interpreter and setup for new dpi. */
-    if (new_magstep != current_magstep) {
-	GhostviewDisableInterpreter(page);
-	reset_size_hints();
-	reset_scroll_bars();
-	changed = True;
-	xdpi = default_xdpi;
-	ydpi = default_ydpi;
-	magnify(&xdpi, new_magstep);
-	magnify(&ydpi, new_magstep);
-							num_args = 0;
-	XtSetFloatArg(args[num_args], XtNxdpi, xdpi);	num_args++;
-	XtSetFloatArg(args[num_args], XtNydpi, ydpi);	num_args++;
-	XtSetValues(page, args, num_args);
-	current_magstep = new_magstep;
-    }
-    return changed;
-}
-
-/* Set new orientation */
-Boolean set_new_orientation(int number)
-{
-    Boolean changed = False;
-    Boolean from_doc = False;
-    XtPageOrientation new_orientation;
-
-    if (app_res.force_orientation) {
-	new_orientation = app_res.orientation;
-    } else {
-	if (doc) {
-	    if (doc->default_page_orientation != NONE) {
-		new_orientation = xorient(doc->default_page_orientation);
-		from_doc = True;
-	    } else if (doc->orientation != NONE) {
-		new_orientation = xorient(doc->orientation);
-		from_doc = True;
-	    } else {
-		new_orientation = app_res.orientation;
-	    }
-	} else {
-	    new_orientation = app_res.orientation;
-	}
-    }
-
-    /* If orientation changed,
-     * stop interpreter and setup for new orientation. */
-    if (new_orientation != current_orientation) {
-	GhostviewDisableInterpreter(page);
-	reset_size_hints();
-	reset_scroll_bars();
-	changed = True;
-	current_orientation = new_orientation;
-    }
-
-    /* mark forced orientation with tie fighter. ("Use the force, Luke") */
-    return changed;
-}
-
 /* Set new pagemedia */
 Boolean set_new_pagemedia(int number)
 {
@@ -1252,7 +990,6 @@ Boolean set_new_pagemedia(int number)
 	(new_urx != current_urx) || (new_ury != current_ury)) {
 	GhostviewDisableInterpreter(page);
 	reset_size_hints();
-	reset_scroll_bars();
 	changed = True;
 	current_llx = new_llx;
 	current_lly = new_lly;
@@ -1268,20 +1005,11 @@ Boolean set_new_pagemedia(int number)
     return changed;
 }
 
-static void build_pagemedia_menu(void)
-{
-    force_document_media = False;
-
-    base_papersize = 0;
-    if (doc) base_papersize = doc->nummedia;
-}
-
 static void new_file(int number)
 {
     Boolean layout_changed = False;
 
     if (setup_ghostview()) layout_changed = True;
-    if (set_new_orientation(number)) layout_changed = True;
     if (set_new_pagemedia(number)) layout_changed = True;
 }
 
