@@ -1,6 +1,6 @@
 /*
    dummy.c
-   Copyright (C) 1997  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2001  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ forking and executing the command.
 #include <X11/Xlib.h>
 #include "../siod/siod.h"
 #include "../common/common.h"
-#include "../common/cmalloc.h"
+#include <Mowitz/MwUtils.h>
 
 extern void execute(char *);
 extern int init_parser(int, char **);
@@ -136,18 +136,18 @@ int plugin_register(char *desc, char *ext, char *cmd, char *name)
         if (n == -1) {  /* allocate room for new plugin */
                 n = nhandler++;
 
-                handler = crealloc(handler, nhandler*sizeof *handler);
+                handler = MwRealloc(handler, nhandler*sizeof *handler);
         } else {        /* reuse old position */
-                cfree(handler[n].desc);
-                cfree(handler[n].ext);
-                cfree(handler[n].cmd);
-		cfree(handler[n].name);
+                MwFree(handler[n].desc);
+                MwFree(handler[n].ext);
+                MwFree(handler[n].cmd);
+		MwFree(handler[n].name);
         }
 
-        handler[n].desc = cstrdup(desc);
-        handler[n].ext = cstrdup(ext);
-        handler[n].cmd = cstrdup(cmd);
-	handler[n].name = cstrdup(name);
+        handler[n].desc = MwStrdup(desc);
+        handler[n].ext = MwStrdup(ext);
+        handler[n].cmd = MwStrdup(cmd);
+	handler[n].name = MwStrdup(name);
         sprintf(b, "%s (*.%s)", desc, ext);
         return 0;
 }
@@ -248,7 +248,7 @@ void mainloop(void)
 	int i;
 
         while (fgets(b, sizeof b, stdin)) {
-                chomp(b);
+                MwChomp(b);
                 for (i = 0; cmds[i].verb; i++) {
                         if (!strncmp(b, cmds[i].verb, strlen(cmds[i].verb)))
                                 break;
