@@ -1,6 +1,6 @@
 /*
    text.c
-   Copyright (C) 1999-2001  Ulric Eriksson <ulric@siag.nu>
+   Copyright (C) 1999-2003  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,9 +23,10 @@
 #include <unistd.h>
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
+#include <X11/Shell.h>
 #include <X11/Xaw/Command.h>
 #include "../common/common.h"
-#include <Mowitz/MwUtils.h>
+#include <Mowitz/Mowitz.h>
 
 static Widget topLevel;
 static Atom plugin_protocol;
@@ -167,8 +168,9 @@ int main(int argc, char **argv)
 	Display *dpy;
 	Widget w;
 
-	topLevel = XtAppInitialize(&ac, "Text",
-		NULL, 0, &argc, argv, NULL, NULL, 0);
+	topLevel = XtOpenApplication(&ac, "Text",
+		NULL, 0, &argc, argv, NULL,
+		mwApplicationShellWidgetClass, NULL, 0);
 	w = XtVaCreateManagedWidget("text",
 		labelWidgetClass, topLevel,
 		XtNlabel, "Text", (char *)0);
