@@ -1,6 +1,6 @@
 /*
    Pathetic Writer
-   Copyright (C) 1997-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,48 +30,16 @@
 
 #include "../common/common.h"
 #include "pw.h"
+#include <Mowitz/MwUtils.h>
 
-static struct {
-	char *name;
-	int value;
-} cchar[] = {
-	{"quot", '"'}, {"amp", '&'}, {"lt", '<'}, {"gt", '>'},
-	{"nbsp", 160}, {"iexcl", 161}, {"cent", 162}, {"pound", 163},
-	{"curren", 164}, {"yen", 165}, {"brvbar", 166}, {"sect", 167},
-	{"uml", 168}, {"copy", 169}, {"ordf", 170}, {"laquo", 171},
-	{"not", 172}, {"shy", 173}, {"reg", 174}, {"macr", 175},
-	{"deg", 176}, {"plusmn", 177}, {"sup2", 178}, {"sup3", 179},
-	{"acute", 180}, {"micro", 181}, {"para", 182}, {"middot", 183},
-	{"cedil", 184}, {"sup1", 185}, {"ordm", 186}, {"raquo", 187},
-	{"frac14", 188}, {"frac12", 189}, {"frac34", 190}, {"iquest", 191},
-	{"Agrave", 192}, {"Aacute", 193}, {"Acirc", 194}, {"Atilde", 195},
-	{"Auml", 196}, {"Aring", 197}, {"AElig", 198}, {"Ccedil", 199},
-	{"Egrave", 200}, {"Eacute", 201}, {"Ecirc", 202}, {"Euml", 203},
-	{"Igrave", 204}, {"Iacute", 205}, {"Icirc", 206}, {"Euml", 207},
-	{"ETH", 208}, {"Ntilde", 209}, {"Ograve", 210}, {"Oacute", 211},
-	{"Ocirc", 212}, {"Otilde", 213}, {"Ouml", 214}, {"times", 215},
-	{"Oslash", 216}, {"Ugrave", 217}, {"Uacute", 218}, {"Ucirc", 219},
-	{"Uuml", 220}, {"Yacute", 221}, {"THORN", 222}, {"szlig", 223},
-	{"agrave", 224}, {"aacute", 225}, {"acirc", 226}, {"atilde", 227},
-	{"auml", 228}, {"aring", 229}, {"aelig", 230}, {"ccedil", 231},
-	{"egrave", 232}, {"eacute", 233}, {"ecirc", 234}, {"euml", 235},
-	{"igrave", 236}, {"iacute", 237}, {"icirc", 238}, {"iuml", 239},
-	{"eth", 240}, {"ntilde", 241}, {"ograve", 242}, {"oacute", 243},
-	{"ocirc", 244}, {"otilde", 245}, {"ouml", 246}, {"divide", 247},
-	{"slash", 248}, {"ugrave", 249}, {"uacute", 250}, {"ucirc", 251},
-	{"uuml", 252}, {"yacute", 253}, {"thorn", 254}, {"yuml", 255},
-	{NULL, 0}
-};
-
-static void save_line(FILE *fp, rich_char *line)
+static void save_line(FILE *fp, MwRichchar *line)
 {
-	int i, c;
+	int c;
+	char b[100];
 	if (!line) return;
 	while ((c = line->c)) {
-		for (i = 0; cchar[i].name; i++)
-			if (cchar[i].value == c) break;
-		if (cchar[i].name) fprintf(fp, "&%s;", cchar[i].name);
-		else putc(c, fp);
+		MwToCchar(b, c);
+		fprintf(fp, b);
 		line++;
 	}
 }
@@ -82,7 +50,7 @@ static int save(char *fn, buffer *buf)
 	FILE *fp;
 	int sty;
 	int s = 0;	/* first sheet only */
-	int i, ll = line_last_used(buf, s), ls = STY_DEFAULT;
+	int i, ll = line_last_used(buf, s), ls = MW_STY_DEFAULT;
 	char *closer = NULL;
 
 	if ((fp = fopen(fn, "w")) == NULL) return 1;
@@ -105,68 +73,68 @@ static int save(char *fn, buffer *buf)
 			closer = NULL;
 		}
 		switch (sty) {
-		case STY_HEADER1:
+		case MW_STY_HEADER1:
 			if (sty != ls) {
 				fprintf(fp, "<H1>\n");
 				closer = "</H1>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_HEADER2:
+		case MW_STY_HEADER2:
 			if (sty != ls) {
 				fprintf(fp, "<H2>\n");
 				closer = "</H2>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_HEADER3:
+		case MW_STY_HEADER3:
 			if (sty != ls) {
 				fprintf(fp, "<H3>\n");
 				closer = "</H3>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_HEADER4:
+		case MW_STY_HEADER4:
 			if (sty != ls) {
 				fprintf(fp, "<H4>\n");
 				closer = "</H4>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_HEADER5:
+		case MW_STY_HEADER5:
 			if (sty != ls) {
 				fprintf(fp, "<H5>\n");
 				closer = "</H5>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_HEADER6:
+		case MW_STY_HEADER6:
 			if (sty != ls) {
 				fprintf(fp, "<H6>\n");
 				closer = "</H6>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_ADDRESS:
+		case MW_STY_ADDRESS:
 			if (sty != ls) {
 				fprintf(fp, "<ADDRESS>\n");
 				closer = "</ADDRESS>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_PREFORMAT:
+		case MW_STY_PREFORMAT:
 			if (sty != ls) {
 				fprintf(fp, "<PRE>\n");
 				closer = "</PRE>";
 			}
 			save_line(fp, buf->sht[s].text[i].p);
 			break;
-		case STY_EMBED:
+		case MW_STY_EMBED:
 			fprintf(fp, "<BR><IMG SRC=\"");
 			save_line(fp, buf->sht[s].text[i].p);
 			fprintf(fp, "\" ALT=\"\"><BR>");
 			break;
-		default:	/* STY_DEFAULT and all STY_USERx styles */
+		default:	/* MW_STY_DEFAULT and all MW_STY_USERx styles */
 			if (ret_bop(buf, s, i))
 				fprintf(fp, "<P>\n");
 			save_line(fp, buf->sht[s].text[i].p);
@@ -191,15 +159,7 @@ static unsigned int row;
 static buffer *buf;
 static int sht;
 
-/* replace these two functions with the ecPrintChar from fileio_rtf.c, which
-   handles line breaking and formats properly
-
-   Even better: don't touch anything until the bop code works. Then just
-   insert everything as it is and rebreak the whole document when we are
-   done. It will be faster and more reliable, and work for the RTF loader
-   as well.
-
-   Store each paragraph on a long line and break it up afterwards. That
+/* Store each paragraph on a long line and break it up afterwards. That
    way we *know* that each line will be a bop, and we only need to change
    the style in tag handlers that deal with such things. Preformatted
    text needs special handling, as usual, but even such text get one
@@ -210,7 +170,6 @@ static void emitchar(int c)
 {
 	if (mute) return;
 
-/*putchar(c);*/
 	if (c == '\n') {	/* break the current line */
 		row++;
 		set_bop(buf, sht, row, 1);
@@ -220,19 +179,17 @@ static void emitchar(int c)
 	}
 }
 
+static void emitstr(char *p)
+{
+	while (*p) emitchar(*p++);
+}
 
 /* &#xxx; => emitchar(atoi(xxx)) */
 static void emitcchar(char *p)
 {
 	int i;
-	if (p[0] == '#') {
-		i = atoi(p+1);
-		if (i >= ' ' && i <= 255) emitchar(atoi(p+1));
-	} else {
-		for (i = 0; cchar[i].name; i++)
-			if (!strcmp(p, cchar[i].name)) break;
-		if (cchar[i].name) emitchar(cchar[i].value);
-	}
+	i = MwFromCchar(p);
+	if (i != -1) emitchar(i);
 }
 
 static void tag_ignore(void)
@@ -271,7 +228,7 @@ static void tag_unmute(void)
 
 static void tag_pre(void)
 {
-	sty = STY_PREFORMAT;
+	sty = MW_STY_PREFORMAT;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 	pre = 1;
@@ -279,7 +236,7 @@ static void tag_pre(void)
 
 static void tag_pre_(void)
 {
-	sty = STY_DEFAULT;
+	sty = MW_STY_DEFAULT;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 	pre = 0;
@@ -287,90 +244,90 @@ static void tag_pre_(void)
 
 static void tag_h1(void)
 {
-	sty = STY_HEADER1;
+	sty = MW_STY_HEADER1;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_h2(void)
 {
-	sty = STY_HEADER2;
+	sty = MW_STY_HEADER2;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_h3(void)
 {
-	sty = STY_HEADER3;
+	sty = MW_STY_HEADER3;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_h4(void)
 {
-	sty = STY_HEADER4;
+	sty = MW_STY_HEADER4;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_h5(void)
 {
-	sty = STY_HEADER5;
+	sty = MW_STY_HEADER5;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_h6(void)
 {
-	sty = STY_HEADER6;
+	sty = MW_STY_HEADER6;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_address(void)
 {
-	sty = STY_ADDRESS;
+	sty = MW_STY_ADDRESS;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_plain(void)
 {
-	sty = STY_DEFAULT;
+	sty = MW_STY_DEFAULT;
 	fmt = style_table[sty].format;
 	emitchar('\n');
 }
 
 static void tag_i(void)
 {
-	sfmt ft;
-	decode_format(fmt, ~0, &ft);
+	MwFmt ft;
+	MwDecodeFormat(fmt, ~0, &ft);
 	ft.italic = 1;
-	fmt = encode_format(~0, &ft);
+	fmt = MwEncodeFormat(~0, &ft);
 }
 
 static void tag_i_(void)
 {
-	sfmt ft;
-	decode_format(fmt, ~0, &ft);
+	MwFmt ft;
+	MwDecodeFormat(fmt, ~0, &ft);
 	ft.italic = 0;
-	fmt = encode_format(~0, &ft);
+	fmt = MwEncodeFormat(~0, &ft);
 }
 
 static void tag_b(void)
 {
-	sfmt ft;
-	decode_format(fmt, ~0, &ft);
+	MwFmt ft;
+	MwDecodeFormat(fmt, ~0, &ft);
 	ft.bold = 1;
-	fmt = encode_format(~0, &ft);
+	fmt = MwEncodeFormat(~0, &ft);
 }
 
 static void tag_b_(void)
 {
-	sfmt ft;
-	decode_format(fmt, ~0, &ft);
+	MwFmt ft;
+	MwDecodeFormat(fmt, ~0, &ft);
 	ft.bold = 0;
-	fmt = encode_format(~0, &ft);
+	fmt = MwEncodeFormat(~0, &ft);
 }
 
 static struct {
@@ -415,6 +372,7 @@ static struct {
 	{"/b", tag_b_},
 	{"head", tag_mute},
 	{"/head", tag_unmute},
+	{"body", tag_unmute},
 	{NULL, NULL}
 };
 
@@ -422,8 +380,11 @@ static struct {
 static void emittag(char *p)
 {
 	int i;
+	for (i = 0; p[i]; i++)
+		if (isspace(p[i])) break;
+	p[i] = '\0';
 	for (i = 0; tag[i].name; i++)
-		if (!cstrcasecmp(p, tag[i].name)) break;
+		if (!MwStrcasecmp(p, tag[i].name)) break;
 	if (tag[i].name) (*tag[i].action)();
 }
 
@@ -466,10 +427,17 @@ static void html_char(int c)
 			state = START;
 		} else if (c == EOF) {
 			errflag = 1;
+			tagbuf[tbi] = '\0';
+			emitstr(tagbuf);
+			tbi = 0;
 			state = END;
+		} else if (isspace(c) || tbi+10 >= sizeof tagbuf) {
+			tagbuf[tbi] = '\0';
+			emitstr(tagbuf);
+			tbi = 0;
+			state = START;
 		} else {
-			if (tbi+10 < sizeof tagbuf)
-				tagbuf[tbi++] = c;
+			tagbuf[tbi++] = c;
 		}
 		break;
 	default:
@@ -496,12 +464,15 @@ static int load(char *fn, buffer *b)
 	row = 1;
 	buf = b;
 	sht = 0;
-	sty = STY_DEFAULT;
+	sty = MW_STY_DEFAULT;
 	fmt = style_table[sty].format;
+	buf->height_interest = 0;
 	while (state != END)
 		html_char(getc(fp));
-	for (row = 1; row <= line_last_used(buf, sht); row++)
+	buf->height_interest = 1;
+	for (row = 1; row <= line_last_used(buf, sht); row++) {
 		rebreak_line(buf, sht, row);
+	}
 	buf->change = 0;
 	fclose(fp);
 	return 0;
@@ -522,7 +493,7 @@ static int myformat(char *fn)
 
         ext = strrchr(fn, '.');
         if (!ext) return 0;     /* no extension */
-        if (cstrcasecmp(ext, ".html") && cstrcasecmp(ext, ".htm"))
+        if (MwStrcasecmp(ext, ".html") && MwStrcasecmp(ext, ".htm"))
                 return 0;       /* wrong extension */
         if ((fp = fopen(fn, "r")) == NULL) return 0;    /* can't open */
         while (fgets(b, sizeof b, fp)) {
@@ -537,7 +508,7 @@ static int myformat(char *fn)
 
 /* ---
 */
-void fileio_html_init()
+void fileio_html_init(void)
 {
 	register_format(load, save, myformat,
 			"Hypertext Markup Language (*.html)");
