@@ -1,6 +1,6 @@
 /*
    Pathetic Writer
-   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 
 #include "pw.h"
 #include "../common/common.h"
-#include "../common/cmalloc.h"
+#include <Mowitz/MwUtils.h>
 
 static int save(char *fn, buffer *buf)
 /* Returns: 0 if successful, otherwise 1 */
@@ -40,9 +40,9 @@ static int save(char *fn, buffer *buf)
 	if (!fp) return 1;
 
 	for (row = 1; row <= to; row++) {
-		char *p = (char *)rc_makeplain(buf->sht[s].text[row].p);
+		char *p = (char *)MwRcMakeplain(buf->sht[s].text[row].p);
 		fprintf(fp, "%s\n", p);
-		cfree(p);
+		MwFree(p);
 	}
 	fclose(fp);
 	return 0;
@@ -82,7 +82,7 @@ static int myformat(char *fn)
 
         ext = strrchr(fn, '.');
         if (!ext) return 0;     /* no extension */
-        if (cstrcasecmp(ext, ".txt"))
+        if (MwStrcasecmp(ext, ".txt"))
                 return 0;       /* wrong extension */
         if ((fp = fopen(fn, "r")) == NULL)
                 return 0;       /* can't open */
@@ -96,7 +96,7 @@ static int myformat(char *fn)
 
 /* ---
 */
-void fileio_txt_init()
+void fileio_txt_init(void)
 {
 	register_format(load, save, myformat, "Text (*.txt)");
 }
