@@ -1,6 +1,6 @@
 /*
    Pathetic Writer
-   Copyright (C) 1997-2002  Ulric Eriksson <ulric@siag.nu>
+   Copyright (C) 1997-2003  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -768,28 +768,37 @@ int page2row(buffer *b, int s, int page)
 	return 1;
 }
 
+char *pack_string(buffer *buf, int s,
+		int r1, int c1, int r2, int c2)
+{
+	unsigned int dummy;
+	return pack_string_area(buf, s, r1, c1, r2, c2, &dummy);
+}
 
 /* ---
 */
 char *pack_string_area(buffer *buf, int s,
 		int r1, int c1, int r2, int c2, unsigned int *size)
 {
-	int n = 0;
-	char *p, *data;
+	int n = 0, m = 0;
+	char *data;
 
-	*size = 32000;
-	p = MwMalloc(*size);
-	data = p;
+	data = MwMalloc(1);
 
 	if (r2 > line_last_used(buf, s)) r2 = line_last_used(buf, s);
-	while (n < 30000 && (r1 < r2 || (r1 == r2 && c1 <= c2))) {
+	while (r1 < r2 || (r1 == r2 && c1 <= c2)) {
 		int c = peek_char(buf, s, r1, c1);
+		if (n >= m) {
+			m += 1000;
+			data = MwRealloc(data, m+1);
+		}
 		data[n++] = c?c:'\n';
 		if (c1 < line_length(buf, s, r1)) c1++;
 		else r1++, c1 = 0;
 	}
 	data[n] = '\0';
-	return p;
+	*size = n;
+	return data;
 }
 
 /* ---
