@@ -1,6 +1,6 @@
 /*
    Pathetic Writer
-   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -185,21 +185,4 @@ position get_mark(window *w)
 	return st[s].mark_pos;
 }
 
-/* ---
-*/
-position set_top(window *w, position p)
-{
-	int s = w->sht;
-	sheet *st = w->buf->sht;
-	return st[s].top = p;
-}
-
-/* ---
-*/
-position get_top(window *w)
-{
-	int s = w->sht;
-	sheet *st = w->buf->sht;
-	return st[s].top;
-}
 
