@@ -1,6 +1,6 @@
 /*
    Pathetic Writer
-   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include "../siod/siod.h"
 
 #include "../common/common.h"
-#include "../common/cmalloc.h"
+#include <Mowitz/MwUtils.h>
 
 #include "pw.h"
 
@@ -43,8 +43,8 @@ int interpreter_count;
 
 static struct {
 	char *name;
-	double (*evalfn)();
-	void (*execfn)();
+	double (*evalfn)(void);
+	void (*execfn)(char *);
 } interpreters[MAX_INTERPRETERS];
 
 char *intnames[MAX_INTERPRETERS];
@@ -64,18 +64,19 @@ int name2interpreter(char *name)
 {
 	int i;
 	for (i = 0; i < interpreter_count; i++)
-		if (!cstrcasecmp(name, interpreters[i].name)) return i;
+		if (!MwStrcasecmp(name, interpreters[i].name)) return i;
 	return -1;
 }
 
 /* ---
 */
-int register_interpreter(char *name, double (*evalfn)(), void (*execfn)())
+int register_interpreter(char *name,
+		double (*evalfn)(void), void (*execfn)(char *))
 {
 	if (interpreter_count >= MAX_INTERPRETERS) return -1;
 	interpreters[interpreter_count].evalfn = evalfn;
 	interpreters[interpreter_count].execfn = execfn;
-	interpreters[interpreter_count].name = cstrdup(name);
+	interpreters[interpreter_count].name = MwStrdup(name);
 	intnames[interpreter_count] = interpreters[interpreter_count].name;
 	intnames[interpreter_count+1] = NULL;
 	return interpreter_count++;
@@ -109,7 +110,7 @@ void va_execute(char *fmt, ...)
 
 /* ---
 */
-void init_interpreters()
+void init_interpreters(void)
 {
 	interpreter_count = 0;
 }
