@@ -1,6 +1,6 @@
 /*
    Siag, Scheme In A Grid
-   Copyright (C) 1996-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1996-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
       functions.
 --- */
 
+#define DEBUG
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -34,16 +36,26 @@
 #include <sys/stat.h>
 
 #include "../siod/siod.h"
-#include "../common/cmalloc.h"
-#include "../common/fonts.h"
+#include <Mowitz/MwUtils.h>
+#include <Mowitz/MwFormat.h>
 #include "../common/common.h"
 #include "../common/plugin.h"
 #include "calc.h"
 
 typedef struct {
 	char *name;
-	 LISP(*function) ();
-} s_fn_table;
+	 LISP(*function) (void);
+} s_fn_table0;
+
+typedef struct {
+	char *name;
+	 LISP(*function) (LISP);
+} s_fn_table1;
+
+typedef struct {
+	char *name;
+	 LISP(*function) (LISP, LISP);
+} s_fn_table2;
 
 /* ---
 */
@@ -55,30 +67,19 @@ static LISP auto_recalc(LISP n)
 	return NIL;
 }
 
+static LISP lcmalloc_stats(void)
+{
+	MwMallocStats();
+	return NIL;
+}
 
 /* ---
 */
-static LISP spawn(LISP command)
+static LISP lspawn(LISP command)
 {
-	char *argv[20], *p, cmd[1024];
-	int argc = 0;
-	pid_t pid;
 	double fl_pid;
 
-	strncpy(cmd, get_c_string(command), 1000);
-	for (p = strtok(cmd, " \t\r\n");
-	     p && argc < 20;
-	     p = strtok(NULL, " \t\r\n")) {
-		argv[argc++] = p;
-	}
-	argv[argc] = NULL;
-	pid = fork();
-	if (pid == 0) {
-		/* this is the child */
-		execvp(argv[0], argv);
-		exit(0);
-	}
-	fl_pid = pid;
+	fl_pid = MwSpawn(get_c_string(command));
 	return flocons(fl_pid);
 }
 
@@ -113,12 +114,14 @@ static LISP load_buffer(void)
 	}
 	strncpy(name, "", 1020);
 	fn[0] = '\0';
-	if (select_file(path, name, loader_patterns, fmt)) {
+	if (select_file(path, name, loader_patterns, fmt, 0)) {
 		sprintf(fn, "%s/%s", path, name);
 
 		b = new_buffer(buffer_name(fn), fn);
 		llpr("Loading");
 
+		set_window_buffer(w_list, b);
+		w_list->sht = 0;
 		if (loadmatrix(fn, b, fmt))
 			llpr("New file");
 	/* this will fail if load returns 1 because the string pool is full */
@@ -131,7 +134,8 @@ static LISP load_buffer(void)
 			char *buttons[] = {"Yes", "No"};
 			int n = alert_box("Load auto startup code?", buttons, 2);
 			if (n != 1) {
-				sprintf(fn, "/tmp/siag%ld", (long)getpid());
+				sprintf(fn, "%s/siag%ld",
+					siag_tmpdir, (long)getpid());
 				sprintf(cmd, "(load \"%s\")", fn);
 				fp = fopen(fn, "w");
 				fwrite(startup, strlen(startup), 1, fp);
@@ -142,9 +146,8 @@ static LISP load_buffer(void)
 			}
 		}
 
-		calc_matrix(b);
 		b->change = FALSE;
-		set_window_buffer(w_list, b);
+		calc_matrix(b);
 		pr_scr_flag = TRUE;
 	}
 	activate_window(w_list);
@@ -155,7 +158,7 @@ static LISP load_buffer(void)
 Load using an external program
 
    1. Ask for the program to use
-   2. Run the program and save output to file in /tmp
+   2. Run the program and save output to file in siag_tmpdir
    3. Load the file using NULL format (i.e. ask for type)
 */
 
@@ -180,8 +183,8 @@ static LISP load_external(void)
 		}
 		while (fgets(b, 250, fp) != NULL && loaders < 20) {
 			if ((p = strtok(b, ":")) && (q = strtok(NULL, "\n"))) {
-				loadname[loaders] = cstrdup(p);
-				loadprog[loaders] = cstrdup(q);
+				loadname[loaders] = MwStrdup(p);
+				loadprog[loaders] = MwStrdup(q);
 				loaders++;
 			}
 		}
@@ -192,7 +195,7 @@ static LISP load_external(void)
 	i = select_from_list("External Program:", loadname, loaders);
 
 	if (i >= 0 && ask_for_str("Parameters:", param)) {
-		sprintf(fn, "/tmp/siag%ld", (long)getpid());
+		sprintf(fn, "%s/siag%ld", siag_tmpdir, (long)getpid());
 		sprintf(cmd, "%s %s > %s", loadprog[i], param, fn);
 		if (system(cmd)) {
 			llpr("External program failed");
@@ -209,6 +212,7 @@ static LISP load_external(void)
 		calc_matrix(b);
 		b->change = FALSE;
 		set_window_buffer(w_list, b);
+		w_list->sht = 0;
 		pr_scr_flag = TRUE;
 	}
 	activate_window(w_list);
@@ -268,7 +272,7 @@ static LISP save_buffer(void)
 	char fn[100];
 	llpr("Saving");
 
-	sprintf(fn, translate("Saved %s"), buf->name);
+	sprintf(fn, _("Saved %s"), buf->name);
 	if (savematrix(buf->path,
 			buf,
 			guess_file_format(buf->path)))
@@ -339,14 +343,14 @@ static LISP save_buffer_as(void)
 	if (p) strncpy(name, p+1, 1020);
 	else strncpy(name, buffer_of_window(w_list)->path, 1020);
 	fn[0] = '\0';
-	if (select_file(path, name, saver_patterns, fmt)) {
+	if (select_file(path, name, saver_patterns, fmt, 1)) {
 		sprintf(fn, "%s/%s", path, name);
 		if (confirm_overwrite && !stat(fn, &sb)) {
 			/* file exists */
 			char q[1024];
 			char *btn[] = {"Yes", "No"};
 			int n;
-			sprintf(q, translate("Overwrite existing %s?"), fn);
+			sprintf(q, _("Overwrite existing %s?"), fn);
 			n = alert_box(q, btn, 2);
 			if (n == 1) return NIL;
 		}
@@ -356,7 +360,7 @@ static LISP save_buffer_as(void)
 		else {
 		        char temp[100];
 			buffer_of_window(w_list)->change = FALSE;
-			sprintf(temp, translate("Saved as %s"), name);
+			sprintf(temp, _("Saved as %s"), name);
 			llpr(temp);
 			
 			strncpy(buffer_of_window(w_list)->path, fn, 1020);
@@ -370,7 +374,7 @@ static LISP save_buffer_as(void)
 Save using an external program
 
    1. Ask for the program to use
-   2. Save to a file in /tmp using NULL format (i.e. ask for type)
+   2. Save to a file in siag_tmpdir using NULL format (i.e. ask for type)
    3. Run the program and read the file as input
 */
 
@@ -394,8 +398,8 @@ static LISP save_external(void)
 		}
 		while (fgets(b, 250, fp) != NULL && savers < 20) {
 			if ((p = strtok(b, ":")) && (q = strtok(NULL, "\n"))) {
-				savename[savers] = cstrdup(p);
-				saveprog[savers] = cstrdup(q);
+				savename[savers] = MwStrdup(p);
+				saveprog[savers] = MwStrdup(q);
 				savers++;
 			}
 		}
@@ -409,7 +413,7 @@ static LISP save_external(void)
 	if (i >= 0 && ask_for_str("Parameters:", param)) {
 		llpr("Saving");
 
-		sprintf(fn, "/tmp/siag%ld", (long)getpid());
+		sprintf(fn, "%s/siag%ld", siag_tmpdir, (long)getpid());
 		if (savematrix(fn, buffer_of_window(w_list), NULL)) {
 			error_box("Couldn't save");
 			return NIL;
@@ -497,12 +501,13 @@ static LISP kill_buffer(LISP x)
 		b = b->next;
 		blist[nblist++] = b->name;
 	} while (b != buffer_of_window(w_list));
-	if ((n = select_from_list("Close Buffer:", blist, nblist)) >= 0) {
+	sprintf(prompt, "%s:", _("Close Buffer"));
+	if ((n = select_from_list(prompt, blist, nblist)) >= 0) {
 		if ((b = find_buffer_by_name(blist[n])) != NULL) {
 			if (b != b->next) {
 				if (b->change && (x != NIL)) {
 					sprintf(prompt,
-						translate("Save %s?"), b->name);
+						_("Save %s?"), b->name);
 					switch (alert_box(prompt,
 							quit_buttons, 3)) {
 					case 0:
@@ -639,7 +644,7 @@ static LISP block_borders(LISP style)
 {
 	buffer *buf = w_list->buf;
 	int sty, r, c, oldfmt, newfmt;
-	sfmt fmt;
+	MwFmt fmt;
 	int s;
 
 	if (NFLONUMP(style))
@@ -648,7 +653,7 @@ static LISP block_borders(LISP style)
 	if  (block_upper(w_list).row < 1 || block_upper(w_list).col < 1)
 		return NIL;
 
-	s = w_list->sht;
+	s = w_list->bsht;
 	for (r = block_upper(w_list).row; r <= block_lower(w_list).row; r++) {
 		cval value;
 		value.number = 0.0;
@@ -658,29 +663,37 @@ static LISP block_borders(LISP style)
 					" ", value, LABEL, s, r, c);
 			}
 			oldfmt = ret_format(buf, s, r, c);
-			decode_format(oldfmt, ~0, &fmt);
+			MwDecodeFormat(oldfmt, ~0, &fmt);
 			switch (sty) {
 			case 1:		/* borders */
 				if (r == block_upper(w_list).row)
-					fmt.borders |= BORDER_TOP;
+					fmt.borders |= MW_BORDER_TOP;
 				if (r == block_lower(w_list).row)
-					fmt.borders |= BORDER_BOTTOM;
+					fmt.borders |= MW_BORDER_BOTTOM;
 				if (c == block_upper(w_list).col)
-					fmt.borders |= BORDER_LEFT;
+					fmt.borders |= MW_BORDER_LEFT;
 				if (c == block_lower(w_list).col)
-					fmt.borders |= BORDER_RIGHT;
+					fmt.borders |= MW_BORDER_RIGHT;
 				break;
 			case 2:		/* grid */
-				fmt.borders |= BORDER_MASK;
+				fmt.borders |= MW_BORDER_MASK;
 				break;
 			case 3:		/* underline */
-				if (r == block_lower(w_list).row)
-					fmt.borders |= BORDER_BOTTOM;
+				fmt.borders ^= MW_BORDER_BOTTOM;
+				break;
+			case 4:		/* border top */
+				fmt.borders ^= MW_BORDER_TOP;
+				break;
+			case 5:		/* border left */
+				fmt.borders ^= MW_BORDER_LEFT;
+				break;
+			case 6:		/* border right */
+				fmt.borders ^= MW_BORDER_RIGHT;
 				break;
 			default:	/* none */
-				fmt.borders &= ~BORDER_MASK;
+				fmt.borders &= ~MW_BORDER_MASK;
 			}
-			newfmt = encode_format(~0, &fmt);
+			newfmt = MwEncodeFormat(~0, &fmt);
 			ins_format(buffer_of_window(w_list), s, r, c, newfmt);
 		}
 	}
@@ -709,7 +722,7 @@ static LISP execute_interpreter_command(LISP intpr)
 	char *intname = get_c_string(intpr);
 	int intp = name2interpreter(intname);
 	if (intp < 0) return NIL;
-	sprintf(prompt, translate("%s command:"), intname);
+	sprintf(prompt, _("%s command:"), intname);
 	b[0] = '\0';
 	if (ask_for_str(prompt, b))
 		exec_expr(intp, b);
@@ -725,17 +738,34 @@ static LISP lalertbox(LISP prompt, LISP buttons)
 
 	while (bno < 10 && NNULLP(buttons)) {
 		char *c = get_c_string(car(buttons));
-		if (c) btext[bno] = cstrdup(c);
-		else btext[bno] = cstrdup("button");
+		if (c) btext[bno] = MwStrdup(c);
+		else btext[bno] = MwStrdup("button");
 		buttons = cdr(buttons);
 		bno++;
 	}
 	if (p == NULL) p = "prompt";
 	n = alert_box(p, btext, bno);
-	while (bno) cfree(btext[--bno]);
+	while (bno) MwFree(btext[--bno]);
 	return flocons(n);
 }
 
+static LISP laboutbox(void)
+{
+	char msg[1024];
+	sprintf(msg, "%s\n\n%s\n\nUlric Eriksson, ulric@siag.nu\n\n%s\n",
+		version,
+		_("A spreadsheet for X"),
+		_("Part of Siag Office"));
+	about_box(msg);
+	return NIL;
+}
+
+static LISP laboutsiag(void)
+{
+	about_siag();
+	return NIL;
+}
+
 /* ---
 */
 static LISP quit_siag(void)
@@ -744,7 +774,7 @@ static LISP quit_siag(void)
 	buffer *b = b_list;
 	do {
 		if (b->change /*&& !plugin*/) {
-			sprintf(prompt, translate("Save %s?"), b->name);
+			sprintf(prompt, _("Save %s?"), b->name);
 			switch (alert_box(prompt, quit_buttons, 3)) {
 			case 0:
 				savematrix(b->path,
@@ -764,7 +794,7 @@ static LISP quit_siag(void)
 	/* remove the temp directory for this process */
 	sprintf(cmd, "rm -rf %s/%ld", siag_basedir, (long)getpid());
 	system(cmd);
-	_exit(0);
+	exit(0);
 	return NIL;
 }
 
@@ -792,8 +822,15 @@ static LISP set_cell_width(void)
 {
 	char b[256];
 	int width = 0;
+	int ccw;
 
 	b[0] = 0;
+
+	/* current cell width as default */
+	ccw = cell_width(buffer_of_window(w_list), w_list->sht,
+	               w_list->buf->sht[w_list->sht].point_pos.col);
+	sprintf(b, "%d", ccw);
+
 	if (ask_for_str("Width:", b))
 		sscanf(b, "%d", &width);
 
@@ -850,8 +887,15 @@ static LISP set_cell_height(void)
 {
 	char b[256];
 	int height = 0;
+	int cch;
 
 	b[0] = 0;
+
+	/* current cell heigth as default */
+	cch = cell_height(buffer_of_window(w_list), w_list->sht,
+			  w_list->buf->sht[w_list->sht].point_pos.row);
+	sprintf(b, "%d", cch);
+
 	if (ask_for_str("Height:", b))
 		sscanf(b, "%d", &height);
 
@@ -881,7 +925,7 @@ static int select_style(void)
 	i = select_from_list("Expression style:", list, n);
 	if (i < 0) s = -1;
 	else s = name2style(list[i]);
-	cfree(list);
+	MwFree(list);
 	return s;
 }
 
@@ -893,7 +937,7 @@ static LISP set_block_style(void)
 	int r, c, r2, c2;
 	buffer *b = buffer_of_window(w_list);
 	int s = w_list->sht;
-	sfmt fmt;
+	MwFmt fmt;
 
 	style = select_style();
 	if (style < 0) return NIL;
@@ -905,11 +949,11 @@ static LISP set_block_style(void)
 		for (c = block_upper(w_list).col;
 			c <= block_lower(w_list).col; c++) {
 			if (c > c2) break;
-			decode_format(ret_format(w_list->buf, s, r, c),
+			MwDecodeFormat(ret_format(w_list->buf, s, r, c),
 					~0, &fmt);
 			fmt.style = style;
 			ins_format(w_list->buf, s, r, c,
-					encode_format(~0, &fmt));
+					MwEncodeFormat(~0, &fmt));
 		}
 	}
 	pr_scr_flag = 1;
@@ -921,7 +965,7 @@ static LISP set_block_style(void)
 static LISP set_cell_style(void)
 {
 	int style;
-	sfmt fmt;
+	MwFmt fmt;
 	int r = get_point(w_list).row;
 	int c = get_point(w_list).col;
 	int s = w_list->sht;
@@ -929,11 +973,11 @@ static LISP set_cell_style(void)
 	style = select_style();
 	if (style < 0) return NIL;
 
-	decode_format(ret_format(w_list->buf, s, r, c),
+	MwDecodeFormat(ret_format(w_list->buf, s, r, c),
 			~0, &fmt);
 	fmt.style = style;
 	ins_format(w_list->buf, s, r, c,
-			encode_format(~0, &fmt));
+			MwEncodeFormat(~0, &fmt));
 
 	pr_scr_flag = 1;
 	return NIL;
@@ -1001,8 +1045,8 @@ static LISP set_data(LISP bname, LISP text, LISP value, LISP type, LISP pos)
 	}
 	if (!buf) return NIL;
 
-	tx = get_c_string(text);
-	if ((i = tx) == NULL) return NIL;
+	i = tx = get_c_string(text);
+	if (i == NULL) return NIL;
 	t = get_c_long(type);
 	switch (t) {
 	case EXPRESSION:
@@ -1284,9 +1328,10 @@ static LISP linput_warp_pointer(LISP value)
 extern int interpreter_count;
 extern char* intnames[];
 
+
 /* ---
 */
-static LISP interpreter_test(LISP intnam, LISP args)
+static LISP interpreter_test(LISP lprompt, LISP intnam, LISP args)
 {
 	buffer *buf = buffer_of_window(w_list);
 	position pos = get_point(w_list);
@@ -1306,8 +1351,12 @@ static LISP interpreter_test(LISP intnam, LISP args)
 	}
 	if (interpreter < 0) return NIL;
 
-	sprintf(prompt, translate("%s expression:"),
-		interpreter2name(interpreter));
+	if (NULLP(lprompt)) {
+		sprintf(prompt, _("%s expression:"),
+			interpreter2name(interpreter));
+	} else {
+		strcpy(prompt, _(get_c_string(lprompt)));
+	}
 Again:
 	if (NULLP(args)) {
 		p = ret_text(buf, s, pos.row, pos.col);
@@ -1324,6 +1373,7 @@ Again:
 		ins_data(buf, interpreter, i, val, type, s, r, c);
 		buf->change = TRUE;
 		calc_matrix(buf);
+		if (ret_type(buf, s, r, c) != ERROR) buf->recalc = 1;
 		pr_scr_flag = TRUE;
 	}
 	if (n > 2) {
@@ -1334,6 +1384,66 @@ Again:
 	return NIL;
 }
 
+static LISP edit_unknown(LISP args)
+{
+	buffer *buf = buffer_of_window(w_list);
+	position pos = get_point(w_list);
+	int type = ERROR;
+	char *prompt = _("Edit:"), b[1024], *p;
+	char *i, *endp;
+	cval val;
+	int n;
+	int s = w_list->sht;
+	int interpreter;
+
+	if (ret_type(buf, s, pos.row, pos.col) == EXPRESSION) {
+		interpreter = ret_interpreter(buf, s, pos.row, pos.col);
+	} else {
+		interpreter =
+		    name2interpreter(
+			get_c_string(
+			    symbol_value(
+				cintern("*standard-interpreter*"),
+				NIL)));
+	}
+	if (interpreter < 0) return NIL;
+
+Again:
+	if (NULLP(args)) {
+		p = ret_text(buf, s, pos.row, pos.col);
+	} else {
+		p = get_c_string(args);
+	}
+	b[0] = 0;
+	if (p) strncpy(b, p, 1023);
+	if ((n = edit_cell(prompt, b))) {
+		int r = pos.row;
+		int c = pos.col;
+		if ((i = b) == 0) return NIL;
+		undo_save(buf, s, r, c, r, c);
+		val.number = strtod(i, &endp);
+		if (endp != i && only_space(endp)) {
+			type = CONSTANT;
+		}
+		ins_data(buf, interpreter, i, val, type, s, r, c);
+		buf->change = TRUE;
+		calc_matrix(buf);
+		if (ret_type(buf, s, r, c) == ERROR) {
+			ins_data(buf, interpreter, i, val, LABEL, s, r, c);
+		} else {
+			buf->recalc = 1;
+		}
+		pr_scr_flag = TRUE;
+	}
+	if (n > 2) {
+		pos.row++;
+		set_point(w_list, pos);
+		goto Again;
+	}
+	return NIL;
+}
+
+
 /* ---
 */
 static LISP lundo_save(LISP r1, LISP c1, LISP r2, LISP c2)
@@ -1408,7 +1518,7 @@ static LISP lpack_area(LISP r1, LISP c1, LISP r2, LISP c2)
 	b = pack_area(w_list->buf, w_list->sht, get_c_long(r1), get_c_long(c1),
 		get_c_long(r2), get_c_long(c2), &size);
 	result = strcons(size, b);
-	cfree(b);
+	MwFree(b);
 	return result;
 }
 
@@ -1421,57 +1531,6 @@ static LISP lunpack_area(LISP b, LISP r, LISP c)
 	return NIL;
 }
 
-/* ---
-*/
-static LISP lregister_x_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
-{
-	return flocons(register_x_font(get_c_string(name),
-					get_c_string(x),
-					get_c_string(xi),
-					get_c_string(xb),
-					get_c_string(xbi)));
-}
-
-/* ---
-*/
-static LISP lregister_ps_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
-{
-	return flocons(register_ps_font(get_c_string(name),
-					get_c_string(x),
-					get_c_string(xi),
-					get_c_string(xb),
-					get_c_string(xbi)));
-}
-
-/* ---
-*/
-static LISP lregister_t1_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
-{
-	return flocons(register_t1_font(get_c_string(name),
-					get_c_string(x),
-					get_c_string(xi),
-					get_c_string(xb),
-					get_c_string(xbi)));
-}
-
-/* ---
-*/
-static LISP lregister_color(LISP name, LISP r, LISP g, LISP b)
-{
-	return flocons(register_color(get_c_string(name),
-					get_c_long(r),
-					get_c_long(g),
-					get_c_long(b)));
-}
-
-/* ---
-*/
-static LISP lfont_alias(LISP alias, LISP name)
-{
-	return flocons(font_alias(get_c_string(alias),
-				get_c_string(name)));
-}
-
 /* ---
 */
 static LISP ladd_sheet(void)
@@ -1574,46 +1633,46 @@ and turns it into an index into the format table.
 */
 static LISP lencode_format(LISP f)
 {
-	sfmt fmt;
+	MwFmt fmt;
 	int result;
-	decode_format(0, ~0, &fmt);	/* put valid fields in fmt */
+	MwDecodeFormat(0, ~0, &fmt);	/* put valid fields in fmt */
 	while (NNULLP(f)) {
 		char *name = get_c_string(CAR(CAR(f)));
-		int mask = fmt_attr2mask(name);
+		int mask = MwFmtAttrToMask(name);
 		LISP value = CDR(CAR(f));
 		f = CDR(f);
 		switch (mask) {
-		case SFMT_FAMILY:
+		case MW_FMT_FAMILY:
 			fmt.family = get_c_string(value);
 			break;
-		case SFMT_SIZE:
+		case MW_FMT_SIZE:
 			fmt.size = get_c_long(value);
 			break;
-		case SFMT_FG:
+		case MW_FMT_FG:
 			fmt.fg = get_c_string(value);
 			break;
-		case SFMT_BG:
+		case MW_FMT_BG:
 			fmt.bg = get_c_string(value);
 			break;
-		case SFMT_BORDERS:
+		case MW_FMT_BORDERS:
 			fmt.borders = get_c_long(value);
 			break;
-		case SFMT_BOLD:
+		case MW_FMT_BOLD:
 			fmt.bold = get_c_long(value);
 			break;
-		case SFMT_ITALIC:
+		case MW_FMT_ITALIC:
 			fmt.italic = get_c_long(value);
 			break;
-		case SFMT_ULINE:
+		case MW_FMT_ULINE:
 			fmt.uline = get_c_long(value);
 			break;
-		case SFMT_HADJ:
+		case MW_FMT_HADJ:
 			fmt.hadj = get_c_long(value);
 			break;
-		case SFMT_VADJ:
+		case MW_FMT_VADJ:
 			fmt.vadj = get_c_long(value);
 			break;
-		case SFMT_STYLE:
+		case MW_FMT_STYLE:
 			fmt.style = name2style(get_c_string(value));
 			break;
 		default:
@@ -1621,7 +1680,7 @@ static LISP lencode_format(LISP f)
 		}
 		
 	}
-	result = encode_format(~0, &fmt);
+	result = MwEncodeFormat(~0, &fmt);
 	return flocons(result);
 }
 
@@ -1644,8 +1703,8 @@ associative list of the format
 */
 static LISP ldecode_format(LISP f)
 {
-	sfmt fmt;
-	decode_format(get_c_long(f), ~0, &fmt);
+	MwFmt fmt;
+	MwDecodeFormat(get_c_long(f), ~0, &fmt);
 	return cons(SS("family", fmt.family),
 		 cons(SL("size", fmt.size),
 		   cons(SS("fg", fmt.fg),
@@ -1669,16 +1728,16 @@ static LISP lselect_attribute(LISP attr)
 	char **list = NULL;
 	int nlist = 0;
 
-	if (!cstrcasecmp(name, "family"))
-     		list = font_list(&nlist);
-	else if (!cstrcasecmp(name, "style"))
+	if (!MwStrcasecmp(name, "family"))
+     		list = MwFontList(&nlist);
+	else if (!MwStrcasecmp(name, "style"))
 		list = style_list(&nlist);
-	else if (!cstrcasecmp(name, "fg") || !cstrcasecmp(name, "bg"))
-		list = color_list(&nlist);
+	else if (!MwStrcasecmp(name, "fg") || !MwStrcasecmp(name, "bg"))
+		list = MwColorList(&nlist);
 
 	if (list) {
 		char prompt[250];
-	   	sprintf(prompt, translate("Change %s:"), name);
+	   	sprintf(prompt, _("Change %s:"), name);
 		choice = select_from_list(prompt, list, nlist);
 	}
 	if (choice < 0) return NIL;
@@ -1705,7 +1764,7 @@ static void insert_plugin(int mode, char *plugin_name)
 	char pn[1024];
 	char cmd[1024];
 	char fmt[80];
-	char *p;
+	char p[1024];
 	buffer *buf = w_list->buf;
 	int n;
 	plugin_t plugin;
@@ -1720,7 +1779,7 @@ static void insert_plugin(int mode, char *plugin_name)
 		else strcpy(name, plugin_name);
 		strcpy(fn, plugin_name);
 	} else {
-		if (!select_file(path, name, plugin_patterns, fmt)) return;
+		if (!select_file(path, name, plugin_patterns, fmt, 0)) return;
 		sprintf(fn, "%s/%s", path, name);
 	}
 
@@ -1731,15 +1790,15 @@ static void insert_plugin(int mode, char *plugin_name)
         plugin.displayed = 0;
 
 	if (mode == PLUGIN_COPY) {
-		plugin.name = cstrdup(pn);
+		plugin.name = MwStrdup(pn);
 		/* copy the file */
-		p = plugin_basedir(buf, NULL);
+		plugin_basedir(p, buf->name);
 		sprintf(pn, "%s/%s", p, plugin.name);
 		sprintf(cmd, "(mkdir %s;cp %s %s)2>/dev/null", p, fn, pn);
 		system(cmd);
 	} else {
 		strcpy(pn, fn);
-		plugin.name = cstrdup(pn);
+		plugin.name = MwStrdup(pn);
 	}
 
 	/* start the plugin */
@@ -1748,7 +1807,7 @@ static void insert_plugin(int mode, char *plugin_name)
 
 	/* prepare the buffer */
 	n = buf->sht[w_list->sht].nplugin++;
-	buf->sht[w_list->sht].plugin = (plugin_t *)crealloc(buf->sht[w_list->sht].plugin,
+	buf->sht[w_list->sht].plugin = (plugin_t *)MwRealloc(buf->sht[w_list->sht].plugin,
 		buf->sht[w_list->sht].nplugin*sizeof(plugin_t));
 	buf->sht[w_list->sht].plugin[n] = plugin;
 	buf->change = TRUE;
@@ -1797,7 +1856,7 @@ static LISP lplugin_select(void)
 static LISP lplugin_export(void)
 {
 	int n;
-	char fn[1024], pn[1024], cmd[1024], path[1024], fmt[80];
+	char p[1024], fn[1024], pn[1024], cmd[1024], path[1024], fmt[80];
 	buffer *buf = w_list->buf;
 	int s = w_list->sht;
 
@@ -1809,10 +1868,11 @@ static LISP lplugin_export(void)
 	/* ask for file name */
 	getcwd(path, 1024);
 	strcpy(fn, buf->sht[s].plugin[n].name);
-	if (!select_file(path, fn, NULL, fmt)) return NIL;
+	if (!select_file(path, fn, NULL, fmt, 1)) return NIL;
 
 	/* save the file */
-	sprintf(pn, "%s/%s", plugin_basedir(buf, NULL), buf->sht[s].plugin[n].name);
+	sprintf(pn, "%s/%s", plugin_basedir(p, buf->name),
+		buf->sht[s].plugin[n].name);
 	sprintf(cmd, "cp %s %s/%s", pn, path, fn);
 	system(cmd);
 	return NIL;
@@ -1830,7 +1890,7 @@ static LISP lplugin_delete(void)
 
 	/* remove plugin */
 	plugin_stop(buf->sht[s].plugin[n].ph);
-	cfree(buf->sht[s].plugin[n].name);
+	MwFree(buf->sht[s].plugin[n].name);
 	buf->sht[s].nplugin--;
 	for (; n < buf->sht[s].nplugin; n++)
 		buf->sht[s].plugin[n] = buf->sht[s].plugin[n+1];
@@ -1951,7 +2011,7 @@ static LISP lpaper_set(LISP b, LISP p)
 	char *n;
 	int w, h, o;
 
-	n = cstrdup(get_c_string(CAR(p)));
+	n = MwStrdup(get_c_string(CAR(p)));
 	w = get_c_long(CAR(CDR(p)));
 	h = get_c_long(CAR(CDR(CDR(p))));
 	o = get_c_long(CAR(CDR(CDR(CDR(p)))));
@@ -2097,8 +2157,8 @@ static LISP lheadfoot_set(LISP b, LISP hf)
 	char *h, *f;
 	int fp;
 
-	h = cstrdup(get_c_string(CAR(hf)));
-	f = cstrdup(get_c_string(CAR(CDR(hf))));
+	h = MwStrdup(get_c_string(CAR(hf)));
+	f = MwStrdup(get_c_string(CAR(CDR(hf))));
 	fp = get_c_long(CAR(CDR(CDR(hf))));
 	if (NULLP(b)) {
 		header = h;
@@ -2196,15 +2256,26 @@ static LISP lmake_backups(LISP p)
 
 static LISP lTRACEME(LISP foo)
 {
-	TRACEME((f, "%s", get_c_string(foo)));
+	fprintf(stderr, "%s\n", get_c_string(foo));
+	MW_TRACE((f, "%s", get_c_string(foo)));
+	return NIL;
+}
+
+static LISP lset_zoom(LISP n)
+{
+	zoom = get_c_double(n);
+	if (zoom < .1) zoom = .1;
+	if (zoom > 10) zoom = 10;
+	pr_scr_flag = 1;
 	return NIL;
 }
 
 /* Commands that take no arguments */
-static s_fn_table fn_table[] =
+static s_fn_table0 fn_table[] =
 {
 	/* moving around */
 	{"go-to", go_to},
+	{"cmalloc_stats", lcmalloc_stats},
 	{"set-standard-width", set_standard_width},
 	{"set-standard-height", set_standard_height},
 	{"set-standard-format", set_standard_format},
@@ -2253,18 +2324,18 @@ static s_fn_table fn_table[] =
 	{"rename-sheet", lrename_sheet},
 	{"get-sheet", lget_sheet},
 
-	{(char *) 0, (LISP(*)())0}
+	{(char *) 0, (LISP(*)(void))0}
 };
 
 /* Commands that take 1 argument */
 
-static s_fn_table fn_table1[] = {
+static s_fn_table1 fn_table1[] = {
 	{"get-cell-width", get_cell_width},
 	{"get-cell-height", get_cell_height},
 	{"auto-recalc", auto_recalc},
 	{"block-borders", block_borders},
 
-	{"spawn", spawn},
+	{"spawn", lspawn},
 	{"execute-interpreter-command", execute_interpreter_command},
 
 	/* low level functions */
@@ -2287,6 +2358,7 @@ static s_fn_table fn_table1[] = {
 	{"respect-prot-get", lrespect_prot_get},
 	{"make-backups", lmake_backups},
 	{"TRACEME", lTRACEME},
+	{"set-zoom", lset_zoom},
 	{NULL, NULL}
 };
 
@@ -2302,7 +2374,8 @@ void init_cmds(void)
 		init_subr_0(fn_table[i].name, fn_table[i].function);
 	for (i = 0; fn_table1[i].name; i++)
 		init_subr_1(fn_table1[i].name, fn_table1[i].function);
-	init_subr_2("interpreter-test", interpreter_test);
+	init_subr_3("interpreter-test", interpreter_test);
+	init_subr_1("edit-unknown", edit_unknown);
 	init_subr_2("downshift-matrix", ldownshift_matrix);
 	init_subr_2("upshift-matrix", lupshift_matrix);
 	init_subr_2("rightshift-matrix", lrightshift_matrix);
@@ -2310,6 +2383,8 @@ void init_cmds(void)
 	init_subr_2("ask-for-str", lask_for_str);
 	init_subr_2("edit-cell", ledit_cell);
 	init_subr_2("alertbox", lalertbox);
+	init_subr_0("aboutbox", laboutbox);
+	init_subr_0("aboutsiag", laboutsiag);
 	init_subr_3("set-format", set_format);
 	init_subr_2("get-format", get_format);
 	init_subr_5("set-data", set_data);
@@ -2319,11 +2394,6 @@ void init_cmds(void)
 	init_subr_2("deletia-add", ldeletia_add);
 	init_subr_4("pack-area", lpack_area);
 	init_subr_3("unpack-area", lunpack_area);
-	init_subr_5("register-x-font", lregister_x_font);
-	init_subr_5("register-ps-font", lregister_ps_font);
-	init_subr_5("register-t1-font", lregister_t1_font);
-	init_subr_2("font-alias", lfont_alias);
-	init_subr_4("register-color", lregister_color);
 	init_subr_3("register-style", lregister_style);
 	init_subr_2("move-sheet", lmove_sheet);
 	init_subr_3("plugin-register", lplugin_register);
@@ -2369,14 +2439,14 @@ static void install(char *k, int l, char *s)
     if (keymap[i].length == l)
       if (!memcmp(k, keymap[i].keys, l)) {
 	if (keymap[i].function != NULL) free(keymap[i].function);
-	keymap[i].function = cstrdup(s);
+	keymap[i].function = MwStrdup(s);
 	return;
       }
   }
   keymap[i].length = l;
-  keymap[i].keys = cmalloc(l);
+  keymap[i].keys = MwMalloc(l);
   for (j = 0; j < l; j++) keymap[i].keys[j] = k[j];
-  keymap[i].function = cstrdup(s);
+  keymap[i].function = MwStrdup(s);
 }
 
 /* ---
@@ -2449,7 +2519,7 @@ static void decode(char *kbd)
 }
 
 /* ---
-static void start_kbd_macro()
+static void start_kbd_macro(void)
 
 Starts recording a keyboard macro.  From now on, everything that is
 typed is also recorded into a buffer.  There is a limit to the size of
@@ -2523,10 +2593,9 @@ static LISP prefix_command(void)
 
 /* ---
 */
-static LISP universal_argument(int argc, char **argv)
+static LISP universal_argument(void)
 {
-  if (argc > 1) argument = atoi(argv[1]);
-  else if (argument) argument *= 4;
+  if (argument) argument *= 4;
   else argument = 4;
   keep_argument = TRUE;
   return NIL;
@@ -2566,7 +2635,7 @@ static LISP add_keybinding(LISP keys, LISP cmd)
 }
 
 
-static s_fn_table calc_fn_table[] = {
+static s_fn_table0 calc_fn_table[] = {
   {"start-kbd-macro", 		start_kbd_macro},
   {"end-kbd-macro",		end_kbd_macro},
   {"call-last-kbd-macro",	call_last_kbd_macro},
@@ -2603,23 +2672,14 @@ void do_cmd(int c)
 {
   char *cmd;
 
-  /* this seems to be stripping the high bit, but I dont think that's right */
-  	if (c & 0x80) {
-    		do_cmd(ESC);
-    		c &= 0x7f;
-  	}
   	if (!is_prefix) prefix_length = 0;
   	prefix_keys[prefix_length++] = c;
   	is_prefix = keep_argument = FALSE;
   	if ((cmd = get_command(prefix_keys, prefix_length))) {
 	  	execute(cmd);
-  	} else if (isdigit(c) || c == '@') {
+	} else {
 		char b[100];
-		sprintf(b, "(edit-expression \"%c\")", c);
-		execute(b);
-  	} else if (isalpha(c)) {
-		char b[100];
-		sprintf(b, "(edit-label \"%c\")", c);
+		sprintf(b, "(edit-unknown \"%c\")", c);
 		execute(b);
 	}
   	if (!keep_argument) argument = 0;
