@@ -1,6 +1,6 @@
 /*
    Siag, Scheme In A Grid
-   Copyright (C) 1996-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1996-2002  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include <ctype.h>
 
 #include "calc.h"
-#include "../common/fonts.h"
+#include <Mowitz/MwFormat.h>
 #include "../common/common.h"
 
 #define START 0
@@ -61,7 +61,7 @@ static int save_csv(char *fn, buffer * buf)
 
 	strcpy(ifs, ",");
 	if (!ask_for_str("Field separator(s):", ifs))
-		return 1;
+		strcpy(ifs, ",");
 
 	if ((fp = fopen(fn, "w")) == (FILE *) 0)
 		return 1;
@@ -90,12 +90,6 @@ static char instring[256];
 static int pi;
 static int sf = 0;
 
-static int only_space(char *p)
-{
-	while (*p && isspace(*p)) p++;
-	return (*p == '\0');
-}
-
 static int guess_type(char *p)
 {
 	char *endp;
@@ -103,7 +97,7 @@ static int guess_type(char *p)
 	if (!strcmp(p, "")) return EMPTY;
 
 	value = strtod(p, &endp);
-	if (only_space(endp)) return EXPRESSION;
+	if (only_space(endp)) return CONSTANT;
 
 	return LABEL;
 }
@@ -117,10 +111,11 @@ static void print_cell(buffer *b, int s, int r, int c, char *p)
 	switch (guess_type(p)) {
 	case EMPTY:
 		break;
-	case EXPRESSION:
+	case CONSTANT:
 		texti = p;
+		value.number = strtod(texti, NULL);
 		ins_data(b, siod_interpreter, texti,
-			value, EXPRESSION, s, r, c);
+			value, CONSTANT, s, r, c);
 		ins_format(b, s, r, c, sf);
 		break;
 	default:	/* anything else is a label */
@@ -212,11 +207,13 @@ static int load_csv(char *fn, buffer *buf)
 	int n;
 
 	strcpy(ifs, ",");
-	if (!ask_for_str("Field separator(s):", ifs))
+	if (!ask_for_str("Field separator(s):", ifs)) {
 		return 1;
+	}
 
-	if ((fp = fopen(fn, "r")) == NULL)
+	if ((fp = fopen(fn, "r")) == NULL) {
 		return 1;
+	}
 
 	n = csv_loader(fp, buf);
 
