@@ -1,6 +1,6 @@
 /*
    Siag, Scheme In A Grid
-   Copyright (C) 1996-2002  Ulric Eriksson <ulric@siag.nu>
+   Copyright (C) 1996-2003  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -309,10 +309,10 @@ void swap_cells(buffer *buf, int s, int r1, int c1, int r2, int c2)
 	spread sp;
 
 	alloc_cell(buf, s, r1, c1);
-        alloc_cell(buf, s, r2, c2);
-        sp = buf->sht[s].matrix[r1][c1];
-        buf->sht[s].matrix[r1][c1] = buf->sht[s].matrix[r2][c2];
-        buf->sht[s].matrix[r2][c2] = sp;
+	alloc_cell(buf, s, r2, c2);
+	sp = buf->sht[s].matrix[r1][c1];
+	buf->sht[s].matrix[r1][c1] = buf->sht[s].matrix[r2][c2];
+	buf->sht[s].matrix[r2][c2] = sp;
 }
 
 /* ---
@@ -777,6 +777,62 @@ void fill_area(buffer *b, int s, int r1, int c1, int r2, int c2)
 	}
 }
 
+char *pack_string(buffer *b, int s, int r1, int c1, int r2, int c2)
+{
+	int r, c, lastr, lastc, i, m;
+	char *p, *data;
+
+	m = 32000;
+	data = MwMalloc(m);
+	lastr = line_last_used(b, s);
+	if (lastr > r2) lastr = r2;
+	i = 0;
+	for (r = r1; r <= lastr; r++) {
+		lastc = col_last_used(b, s, r);
+		if (lastc > c2) lastc = c2;
+		for (c = c1; c <= lastc; c++) {
+			p = ret_pvalue(NULL, b, s, r, c, -1);
+			if (i+strlen(p)+10 > m) {
+				m = i+strlen(p)+32000;
+				data = MwRealloc(data, m);
+			}
+			strcpy(data+i, p);
+			i += strlen(p);
+			if (c < c2) data[i++] = ',';
+		}
+		data[i++] = '\n';
+	}
+	data[i] = '\0';
+	return data;
+}
+
+void unpack_string(buffer *b, char *data, int s, int row, int col)
+{
+	char *p;
+	int n = 0;
+	cval value;
+	value.number = 0;
+
+	for (p = data; *p; p++)
+		if (*p == '\n') n++;
+
+	undo_save(b, s, row, col, row+n, col);
+
+	while ((p = strchr(data, '\n')) != NULL) {
+		*p = '\0';
+		ins_data(b, siod_interpreter, data,
+			value, LABEL, s, row, col);
+		row++;
+		data = p+1;
+	}
+	if (*data) {
+		ins_data(b, siod_interpreter, data,
+			value, LABEL, s, row, col);
+	}
+	pr_scr_flag = TRUE;
+	show_cur(w_list);
+}
+
 /* ---
 Allocate a block of memory of sufficient size and encode the contents
    of a specified rectangular area into it. Caller must free.
@@ -785,55 +841,55 @@ Allocate a block of memory of sufficient size and encode the contents
 char *pack_area(buffer *b, int s, int r1, int c1, int r2, int c2,
 	unsigned int *size)
 {
-	int r, c;
+	int r, c, i;
 	int intp;
-	char *p, *data;
+	char *data;
 
 	/* use fixed size buffer for now */
 	*size = 32000;
-	p = MwMalloc(*size);
-	data = p;
+	data = MwMalloc(*size);
 	sprintf(data, "%d %d", r2-r1+1, c2-c1+1);
-	data += strlen(data)+1;
+	i = strlen(data)+1;
 
 	for (r = r1; r <= r2 && r <= line_last_used(b, s); r++) {
 		for (c = c1; c <= c2 && c <= col_last_used(b, s, r); c++) {
 			switch (ret_type(b, s, r, c)) {
 			case EMPTY:
-				sprintf(data, "#");
+				sprintf(data+i, "#");
 				break;
 			case LABEL:
-				sprintf(data, "\"%s",
+				sprintf(data+i, "\"%s",
 					ret_text(b, s, r, c));
 				break;
 			case CONSTANT:
-				sprintf(data, "=%s",
+				sprintf(data+i, "=%s",
 					ret_text(b, s, r, c));
 				break;
 			case EMBED:
-				sprintf(data, "m%s",
+				sprintf(data+i, "m%s",
 					ret_text(b, s, r, c));
 				break;
 			default:
 				intp = ret_interpreter(b, s, r, c);
-				sprintf(data, "+%s,%s",
+				sprintf(data+i, "+%s,%s",
 					interpreter2name(intp),
 					ret_text(b, s, r, c));
 				break;
 			}
-			data += strlen(data)+1;
-			if (data-p > 31000) {
-				MwFree(p);
+			i += strlen(data+i)+1;
+			if (i+2000 > *size) {
+				*size += 32000;
+				data = MwRealloc(data, *size);
 				return NULL;
 			}
 		}
 		/* use newline to mark end of line */
-		sprintf(data, "\n");
-		data += strlen(data)+1;
+		sprintf(data+i, "\n");
+		i += strlen(data+i)+1;
 	}
 	/* use null character to mark end of area */
-	*data = '\0';
-	return p;
+	*(data+i) = '\0';
+	return data;
 }
 
 /* ---
