@@ -37,6 +37,7 @@
 #include <time.h>
 #include <signal.h>
 #include <sys/wait.h>
+#include <Mowitz/MwUtils.h>
 
 #include "../common/common.h"
 
@@ -67,7 +68,7 @@ static cval parse_python_expr(buffer *b, char *expr, int s, int row, int col)
 	siag_buffer = b;
 	siag_sht = s;
 
-TRACEME((stderr, "Python expression: '%s'\n", expr));
+MW_TRACE((stderr, "Python expression: '%s'\n", expr));
 
 	PyErr_Clear();
 	result = PyRun_String(expr, Py_eval_input, globals, globals);
@@ -75,27 +76,27 @@ TRACEME((stderr, "Python expression: '%s'\n", expr));
 	if (result == NULL) {
 		siag_type = ERROR;
 		siag_result.number = 0;
-TRACEME((stderr, "Error\n"));
+MW_TRACE((stderr, "Error\n"));
 	} else if (PyFloat_Check(result)) {
 		siag_type = EXPRESSION;
 		siag_result.number = PyFloat_AsDouble(result);
-TRACEME((stderr, "Float = %f\n", siag_result.number));
+MW_TRACE((stderr, "Float = %f\n", siag_result.number));
 	} else if (PyLong_Check(result)) {
 		siag_type = EXPRESSION;
 		siag_result.number = PyLong_AsLong(result);
-TRACEME((stderr, "Long = %f\n", siag_result.number));
+MW_TRACE((stderr, "Long = %f\n", siag_result.number));
 	} else if (PyNumber_Check(result)) {
 		siag_type = EXPRESSION;
 		siag_result.number = PyInt_AsLong(result);
-TRACEME((stderr, "Integer = %f\n", siag_result.number));
+MW_TRACE((stderr, "Integer = %f\n", siag_result.number));
 	} else if (PyString_Check(result)) {
 		siag_type = STRING;
 		siag_result.text = PyString_AsString(result);
-TRACEME((stderr, "String = '%s'\n", siag_result.text));
+MW_TRACE((stderr, "String = '%s'\n", siag_result.text));
 	} else {
 		siag_type = ERROR;
 		siag_result.number = 0;
-TRACEME((stderr, "Bogus value\n"));
+MW_TRACE((stderr, "Bogus value\n"));
 	}
 
 	errorflag = 0;
