@@ -26,12 +26,14 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <stdlib.h>
+#include <stdio.h>
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 #include <X11/Xos.h>
+#include <Mowitz/Mowitz.h>
 #include "GhostviewP.h"
 #include <ctype.h>
 
@@ -78,6 +80,8 @@ extern int errno;
 #define GV_BUFSIZ 1024
 #endif /* VMS */
 
+static int antialiasing = 1;
+
 static void ComputeXdpi();
 static void ComputeYdpi();
 
@@ -408,7 +412,7 @@ Input(client_data, source, id)
 		struct record_list *ps_old = gvw->ghostview.ps_input;
 		gvw->ghostview.ps_input = ps_old->next;
 		if (ps_old->close) fclose(ps_old->fp);
-		XtFree((char *)ps_old);
+		MwFree((char *)ps_old);
 	    }
 
 	    /* Have to seek at the beginning of each section */
@@ -621,9 +625,9 @@ Destroy(w)
 
     StopInterpreter(w);
     XtReleaseGC(w, gvw->ghostview.gc);
-    if (gvw->ghostview.input_buffer) XtFree(gvw->ghostview.input_buffer);
+    if (gvw->ghostview.input_buffer) MwFree(gvw->ghostview.input_buffer);
 #ifdef VMS
-    if (gvw->ghostview.output_buffer) XtFree(gvw->ghostview.output_buffer);
+    if (gvw->ghostview.output_buffer) MwFree(gvw->ghostview.output_buffer);
 #endif /* VMS */
     if (gvw->core.background_pixmap != XtUnspecifiedPixmap)
 	XFreePixmap(XtDisplay(w), gvw->core.background_pixmap);
@@ -1120,12 +1124,12 @@ StartInterpreter(w)
     if (gvw->ghostview.disable_start) return;
 
     argv[argc++] = gvw->ghostview.interpreter;
-    argv[argc++] = "-sDEVICE=x11";
+    argv[argc++] = (antialiasing)?"-sDEVICE=x11alpha":"-sDEVICE=x11";
     argv[argc++] = "-dNOPAUSE";
     if (gvw->ghostview.quiet) argv[argc++] = "-dQUIET";
     if (gvw->ghostview.safer) argv[argc++] = "-dSAFER";
     if (gvw->ghostview.arguments) {
-	cptr = arguments = XtNewString(gvw->ghostview.arguments);
+	cptr = arguments = MwStrdup(gvw->ghostview.arguments);
 	while (isspace(*cptr)) cptr++;
 	while (*cptr) {
 	    argv[argc++] = cptr;
@@ -1223,7 +1227,7 @@ StartInterpreter(w)
 	    XtAppAddInput(XtWidgetToApplicationContext(w), std_err[0],
 			  (XtPointer)XtInputReadMask, Output, (XtPointer)w);
     }
-    if (arguments) XtFree(arguments);
+    if (arguments) MwFree(arguments);
 }
 
 /* Stop the interperter, if present, and remove any Input sources. */
@@ -1249,7 +1253,7 @@ StopInterpreter(w)
 	    struct record_list *ps_old = gvw->ghostview.ps_input;
 	    gvw->ghostview.ps_input = ps_old->next;
 	    if (ps_old->close) fclose(ps_old->fp);
-	    XtFree((char *)ps_old);
+	    MwFree((char *)ps_old);
 	}
     }
     if (gvw->ghostview.interpreter_output >= 0) {
@@ -1282,6 +1286,24 @@ InterpreterFailed(w)
  *	Public Routines
  */
 
+/* GhostviewSetAntialiasing:
+ */
+void
+GhostviewSetAntialiasing(w)
+    Widget w;
+{
+    antialiasing = True;
+}
+
+/* GhostviewUnsetAntialiasing:
+ */
+void
+GhostviewUnsetAntialiasing(w)
+    Widget w;
+{
+    antialiasing = False;
+}
+
 /* GhostviewDisableInterpreter:
  * Stop any interpreter and disable new ones from starting.
  */
@@ -1372,7 +1394,7 @@ GhostviewSendPS(w, fp, begin, len, close)
     struct record_list *ps_new;
 
     if (gvw->ghostview.interpreter_input < 0) return False;
-    ps_new = (struct record_list *) XtMalloc(sizeof (struct record_list));
+    ps_new = (struct record_list *) MwMalloc(sizeof (struct record_list));
     ps_new->fp = fp;
     ps_new->begin = begin;
     ps_new->len = len;
@@ -1381,7 +1403,7 @@ GhostviewSendPS(w, fp, begin, len, close)
     ps_new->next = NULL;
 
     if (gvw->ghostview.input_buffer == NULL) {
-	gvw->ghostview.input_buffer = XtMalloc(GV_BUFSIZ);
+	gvw->ghostview.input_buffer = MwMalloc(GV_BUFSIZ);
     }
 
     if (gvw->ghostview.ps_input == NULL) {
@@ -1638,7 +1660,7 @@ GLInsert(w)
     ** Insert this widget after the list head
     */
     first = (GL->next == NULL);
-    new = XtMalloc(GLI_Size);
+    new = MwMalloc(GLI_Size);
     new->w = w;
     new->next = GL->next;
     GL->next = new;
@@ -1667,7 +1689,7 @@ GLRemove(w)
     }
     if(cur){
        prev->next = cur->next;
-       XtFree(cur);
+       MwFree(cur);
        last = (GL->next == NULL);
     }
     /*
@@ -1705,7 +1727,7 @@ Input(gvw)
 	    struct record_list *ps_old = gvw->ghostview.ps_input;
 	    gvw->ghostview.ps_input = ps_old->next;
 	    if (ps_old->close) fclose(ps_old->fp);
-	    XtFree((char *)ps_old);
+	    MwFree((char *)ps_old);
 	}
 	if(gvw->ghostview.ps_input){
 	    /* Have to seek at the beginning of each section */
@@ -1844,7 +1866,11 @@ StartInterpreter(w)
     ** Build Ghostscript startup command
     */
     strcpy(cmd, gvw->ghostview.interpreter);
-    strcat(cmd, " \"-sDEVICE=x11\" \"-dNOPAUSE\" ");
+    if (antialiasing)
+        strcat(cmd, " \"-sDEVICE=x11alpha\"");
+    else
+        strcat(cmd, " \"-sDEVICE=x11\"");
+    strcat(cmd, " \"-dNOPAUSE\" ");
     if (gvw->ghostview.quiet) strcat(cmd, "\"-dQUIET\" ");
     if (gvw->ghostview.safer) strcat(cmd, "\"-dSAFER\" ");
     if (gvw->ghostview.arguments) {
@@ -1922,7 +1948,7 @@ StartInterpreter(w)
     }
     gvw->ghostview.interpreter_output = ch2;
     if (gvw->ghostview.output_buffer == NULL) {
-	gvw->ghostview.output_buffer = XtMalloc(GV_BUFSIZ);
+	gvw->ghostview.output_buffer = MwMalloc(GV_BUFSIZ);
     }
     GLInsert(gvw);
     /*
@@ -1954,7 +1980,7 @@ StopInterpreter(w)
 	    struct record_list *ps_old = gvw->ghostview.ps_input;
 	    gvw->ghostview.ps_input = ps_old->next;
 	    if (ps_old->close) fclose(ps_old->fp);
-	    XtFree((char *)ps_old);
+	    MwFree((char *)ps_old);
 	}
     }
     if (gvw->ghostview.interpreter_output >= 0) {
@@ -1992,7 +2018,7 @@ GhostviewSendPS(w, fp, begin, len, close)
 
     if (gvw->ghostview.interpreter_input < 0) return False;
     if(len != 0){
-	ps_new = (struct record_list *) XtMalloc(sizeof (struct record_list));
+	ps_new = (struct record_list *) MwMalloc(sizeof (struct record_list));
 	ps_new->fp = fp;
 	ps_new->begin = begin;
 	ps_new->len = len;
@@ -2001,7 +2027,7 @@ GhostviewSendPS(w, fp, begin, len, close)
 	ps_new->next = NULL;
 
 	if (gvw->ghostview.input_buffer == NULL) {
-	    gvw->ghostview.input_buffer = XtMalloc(GV_BUFSIZ);
+	    gvw->ghostview.input_buffer = MwMalloc(GV_BUFSIZ);
 	}
 
 	if (gvw->ghostview.ps_input == NULL) {
