@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1996-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+    Copyright (C) 1996-2001  Ulric Eriksson <ulric@siag.nu>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -90,6 +90,7 @@ The typical use of the plugin library will be like this:
 #include <string.h>
 #include <ctype.h>
 #include <signal.h>
+#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -99,7 +100,7 @@ The typical use of the plugin library will be like this:
 #include <X11/cursorfont.h>
 #include <X11/xpm.h>
 #include "../common/common.h"
-#include "../common/cmalloc.h"
+#include <Mowitz/Mowitz.h>
 #include "plugin.h"
 
 #define REPARENT_LOOPS 50
@@ -108,8 +109,6 @@ The typical use of the plugin library will be like this:
 /*#define DEBUG
 */
 
-#include "../common/traceme.h"
-
 char **plugin_patterns;
 
 static struct {
@@ -183,18 +182,18 @@ int plugin_read(int ph, char *b)
 static char **parse_cmd(char *cmd, char *fn, char *name)
 {
 	int i = 0;
-	char **argv = (char **)cmalloc(10*sizeof(char *));
-	char *cmd_copy = cstrdup(cmd);
+	char **argv = (char **)MwMalloc(10*sizeof(char *));
+	char *cmd_copy = MwStrdup(cmd);
 	char *p;
 
 	for (p = strtok(cmd_copy, " \n"); p && i < 9; p = strtok(NULL, " \n")) {
 		if (p[0] == '%') {
 			switch (p[1]) {
 			case 's':
-				argv[i++] = cstrdup(fn);
+				argv[i++] = MwStrdup(fn);
 				break;
 			case 'W':
-				argv[i++] = cstrdup(name);
+				argv[i++] = MwStrdup(name);
 				break;
 			case '%':
 				argv[i++] = p+1;
@@ -421,7 +420,7 @@ char *plugin_help(int ph)
 	plugin_write(ph, "HELP\n");
 	cmd[0] = '\0';
 	plugin_read(ph, cmd);
-	return cstrdup(cmd);
+	return MwStrdup(cmd);
 }
 
 /* ---
@@ -505,11 +504,6 @@ int plugin_show(int ph, Widget grid)
                 XReparentWindow(display, w, to, 0, 0);
                 XQueryTree(display, w, &root, &parent, &child, &n);
                 XSync(display, False);
-#if 0
-                if (parent == to) {
-                        break;
-                }
-#endif
         }
 	XMoveWindow(display, w, 2, 2);
         XMapWindow(display, w);
@@ -560,9 +554,6 @@ int plugin_hide(int ph)
 		Window root, parent, *child;
 		Cardinal n;
 		XQueryTree(display, w, &root, &parent, &child, &n);
-#if 0
-		if (parent == root) break;
-#endif
 		XReparentWindow(display, w, root, 0, 0);
 		XSync(display, False);
 	}
@@ -615,10 +606,10 @@ static void prnt(Pixmap pixmap, FILE *fp)
         }
         sscanf(data[0], "%d %d %d %d", &width, &height, &ncolors, &cpp);
 
-        cm = (colors *)cmalloc(ncolors*sizeof(colors));
+        cm = (colors *)MwMalloc(ncolors*sizeof(colors));
 
         for (i = 0; i < ncolors; i++) {
-                cm[i].chars = (char *)cmalloc(cpp);
+                cm[i].chars = (char *)MwMalloc(cpp);
                 memcpy(cm[i].chars, data[i+1], cpp);
                 key = strtok(data[i+1]+cpp, " \t");     /* skip past chars */
                 color = strtok(NULL, " \t");
@@ -681,14 +672,9 @@ static void prnt(Pixmap pixmap, FILE *fp)
         fprintf(fp, "%%\n\n");
         fprintf(fp, "grestore\n");
 
-#if 0	/* and why not? */
-	fprintf(fp, "%%%%Trailer\n");
-	fprintf(fp, "%%%%EOF\n");
-#endif
-
-        for (i = 0; i < ncolors; i++) cfree(cm[i].chars);
-        cfree(cm);
-        cfree(data);
+        for (i = 0; i < ncolors; i++) MwFree(cm[i].chars);
+        MwFree(cm);
+        MwFree(data);
 }
 
 /* ---
@@ -808,22 +794,22 @@ int plugin_register(char *desc, char *ext, char *cmd)
 	if (n == -1) {	/* allocate room for new plugin */
 		n = nhandler++;
 
-		handler = crealloc(handler, nhandler*sizeof *handler);
-		plugin_patterns = crealloc(plugin_patterns,
+		handler = MwRealloc(handler, nhandler*sizeof *handler);
+		plugin_patterns = MwRealloc(plugin_patterns,
 			(nhandler+1)*sizeof *plugin_patterns);
 		plugin_patterns[nhandler] = NULL;
 	} else {	/* reuse old position */
-		cfree(handler[n].desc);
-		cfree(handler[n].ext);
-		cfree(handler[n].cmd);
-		cfree(plugin_patterns[n]);
+		MwFree(handler[n].desc);
+		MwFree(handler[n].ext);
+		MwFree(handler[n].cmd);
+		MwFree(plugin_patterns[n]);
 	}
 
-	handler[n].desc = cstrdup(desc);
-	handler[n].ext = cstrdup(ext);
-	handler[n].cmd = cstrdup(cmd);
+	handler[n].desc = MwStrdup(desc);
+	handler[n].ext = MwStrdup(ext);
+	handler[n].cmd = MwStrdup(cmd);
 	sprintf(b, "%s (*.%s)", desc, ext);
-	plugin_patterns[n] = cstrdup(b);
+	plugin_patterns[n] = MwStrdup(b);
 	return 0;
 }
 
@@ -983,9 +969,6 @@ static void PluginDispatchEvent(Widget w, XtPointer pt, XEvent *e, Boolean *b)
 	int n, ph, ticket;
 	char cmd[1024];
 	if (!plugin_message(e)) {
-#if 0
-		fprintf(stderr, "Not a plugin message\n");
-#endif
 		return;
 	}
 	if (e->xclient.data.l[0] != 0) {
