@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1996-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+    Copyright (C) 1996-2001  Ulric Eriksson <ulric@siag.nu>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,23 +17,11 @@
     MA 02111-1307, USA.
 */
  
-extern String label_get(Widget);
-extern void label_set(Widget, String);
-extern Boolean state_get(Widget);
-extern void state_set(Widget, Boolean, int, int);
-extern Dimension width_get(Widget);
-extern Dimension height_get(Widget);
-
 extern void start_splash(void);
 extern void stop_splash(void);
-extern Pixmap load_pixmap(Widget, char *);
-
-/* tooltip.c */
-#if 0
-#define TOOLTIP_NONE 0
-#define TOOLTIP_LABEL 1
-#define TOOLTIP_POPUP 2
-#endif
-extern void tooltip_init(Widget, unsigned long, unsigned long);
-extern void tooltip_mode_set(int, Widget);
+extern Pixmap load_pixmap(Display *, Pixel, char *);
+extern void free_colors(void);
+extern int xerror_handler(Display *, XErrorEvent *);
+extern void theme_init(Display *);
+extern int select_theme(Widget);
 
