@@ -435,11 +435,12 @@ caddr_t closure, calldata;
 					show_position(position);
 				}
 			}
-			if (answer == NO)
+			if (answer == NO) {
 				if (direction == XawsdRight)
 					show_position(position + searchblock.length);
 				else
 					show_position(position);
+			}
 			if (answer == CANCEL) {
 				XtPopdown(veto_popup);
 				return;
