@@ -1,6 +1,7 @@
 /*
 
    Copyright (c) 1995  Randolf Werner
+   Changes 1998-2003 by Ulric Eriksson
 
    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files (the
@@ -205,9 +206,11 @@ static void ActionCallMenu(Widget widget, XEvent * event,
 	if (*n_params != 1) {
 		return;
 	}
-	menuwidget = XtNameToWidget(top, params[0]);
+	menuwidget = XtNameToWidget(menubox, params[0]);
 	if (menuwidget != NULL) {
 		XtCallCallbacks(menuwidget, XtNcallback, NULL);
+	} else {
+		printf("xedCallMenu(%s) unknown\n", *params);
 	}
 }
 
@@ -790,6 +793,7 @@ static void create_toolbar(Widget pw)
 		     "Print the file");
 	make_vsep(pw);
 	make_command(DoEditCut, pw, "cut.xpm", "Cut");
+	make_command(DoEditCopy, pw, "copy.xpm", "Copy");
 	make_command(DoEditPaste, pw, "paste.xpm", "Paste");
 	modelabel = make_command(xedtoggleoverwrite, pw, "insert.xpm",
 			"Toggle overwrite/insert");
@@ -924,7 +928,8 @@ char *filename;
 		mwMenuWidgetClass, menubox, NULL, 0);
 
 	add_menu_entry(menu, "cut", "Cut", DoEditCut, NULL);
-	add_menu_entry(menu, "paste", "Paste", DoEditCut, NULL);
+	add_menu_entry(menu, "copy", "Copy", DoEditCopy, NULL);
+	add_menu_entry(menu, "paste", "Paste", DoEditPaste, NULL);
 	add_menu_entry(menu, "right", "Shift Selection Right",
 		       DoEditShiftSelRight, NULL);
 	add_menu_entry(menu, "left", "Shift Selection Left",
@@ -1165,11 +1170,12 @@ int main(int argc, char **argv)
 
 	common_init("XedPlus %s. No Warranty.");
 
-	top = XtVaAppInitialize(&app_context,
+	top = XtVaOpenApplication(&app_context,
 				"XedPlus",
 				NULL, 0,
 				&argc, argv,
 				fallback_resources,
+				mwApplicationShellWidgetClass,
 				(char *) 0);
 
 	theme_init(XtDisplay(top));
