@@ -4,6 +4,7 @@
  (c) S.Marlow 1990-92
  (c) A.Graef 1994
  (c) R.Vogelgesang 1994 (`Xfm.BourneShells' stuff)
+ (c) Ulric Eriksson 1998-2003
 
  main module for file manager    
 ---------------------------------------------------------------------------*/
@@ -424,9 +425,10 @@ int main(int argc, char *argv[])
 	common_init("Xfiler %s. No Warranty.");
 
   /* initialise the application and create the application shell */
-  toplevel = XtAppInitialize(&app_context, "Xfiler", options, XtNumber(options),
-			     &argc, argv, fallback_resources, shell_args,
-			     XtNumber(shell_args) );
+  toplevel = XtOpenApplication(&app_context, "Xfiler", options, XtNumber(options),
+			     &argc, argv, fallback_resources,
+			     mwApplicationShellWidgetClass,
+			     shell_args, XtNumber(shell_args) );
 
   theme_init(XtDisplay(toplevel));
 
