@@ -11,21 +11,35 @@
   and functions for desensetising and 'ticking' menu entries.
 -----------------------------------------------------------------------------*/
 
+#include <stdio.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#include "../xcommon/SiagMenuButton.h"
-#include <X11/Xaw/SimpleMenu.h>
-#include <X11/Xaw/SmeLine.h>
-#include <X11/Xaw/SmeBSB.h>
+
+#include <Mowitz/Mowitz.h>
+
 #include <X11/Xaw/Command.h>
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Label.h>
 #include <X11/Xaw/Box.h>
-#include "../xcommon/TextField.h"
 
-#include "../common/common.h"
 #include "Files.h"
+#include "../common/common.h"
+
+#include "blank.xpm"
+#include "dot.xpm"
+
+static Icon *dot_bm, *blank_bm;
+
+static void load_icons(void)
+{
+	int loaded = 0;
+	if (!loaded) {
+		blank_bm = X_XpmDataToIcon(XtDisplay(toplevel), blank_xpm);
+		dot_bm = X_XpmDataToIcon(XtDisplay(toplevel), dot_xpm);
+		loaded = 1;
+	}
+}
 
 #define PADDING 20
 #define TEXT_WIDTH 350
@@ -103,18 +117,17 @@ Widget *createFloatingMenu(String menu_name,
   
   item_widgets = (Widget *) XtMalloc(n_items * sizeof(Widget));
 
-  *menu_widget = XtCreatePopupShell(menu_name, simpleMenuWidgetClass,
+  *menu_widget = XtCreatePopupShell(menu_name, mwMenuWidgetClass,
 				    parent, NULL, 0 );
 
   for (i = 0; i < n_items; i++) {
     if (items[i].callback == NULL)
-      XtCreateManagedWidget(items[i].item_name, smeLineObjectClass,
+      XtCreateManagedWidget(items[i].item_name, mwLineMEObjectClass,
 			    *menu_widget, NULL, 0 );
     else {
       item_widgets[i] = XtVaCreateManagedWidget(items[i].item_name,
-			      smeBSBObjectClass, *menu_widget,
-			      XtNlabel, translate(items[i].item_label),
-			      XtNleftMargin, left_margin,
+			      mwLabelMEObjectClass, *menu_widget,
+			      XtNlabel, _(items[i].item_label),
 			      (char *)0);
       XtAddCallback(item_widgets[i], XtNcallback,
 		    (XtCallbackProc) items[i].callback, client_data );
@@ -144,26 +157,33 @@ Widget *createMenu(String menu_name, String menu_label, MenuItemList items,
 {
   int i;
   Widget menu_widget, button_widget, *item_widgets;
+  char buf[80], *mnu;
+
+  sprintf(buf, "%s_menu", menu_name);
+  mnu = MwStrdup(buf);
   
   item_widgets = (Widget *) XtMalloc(n_items * sizeof(Widget));
 
   button_widget = XtVaCreateManagedWidget(menu_name,
-		siagMenuButtonWidgetClass, parent,
-		XtNlabel, translate(menu_label),
+		mwMBButtonObjectClass, parent,
+		XtNlabel, _(menu_label),
+		XtNmenu_name, mnu,
+		XtNgravitation, (strcmp(menu_label, "Help")?XtCleft:XtCright),
+		(char *)0);
+  menu_widget = XtVaCreatePopupShell(mnu,
+		mwMenuWidgetClass, parent,
 		(char *)0);
-  menu_widget = XtCreatePopupShell( "menu", simpleMenuWidgetClass,
-      button_widget, NULL, 0 );
 
   for (i = 0; i < n_items; i++) {
-    if (items[i].callback == NULL)
-      XtCreateManagedWidget(items[i].item_name, smeLineObjectClass,
-			    menu_widget, NULL, 0 );
-    else {
+    if (items[i].callback == NULL) {
+      XtVaCreateManagedWidget(items[i].item_name,
+		mwLineMEObjectClass, menu_widget,
+		(char *)0);
+    } else {
       item_widgets[i] = XtVaCreateManagedWidget(items[i].item_name,
-				smeBSBObjectClass, menu_widget,
-				XtNlabel, translate(items[i].item_label),
-				XtNleftMargin, left_margin,
-				(char *)0);
+		mwLabelMEObjectClass, menu_widget,
+		XtNlabel, _(items[i].item_label),
+		(char *)0);
 
       XtAddCallback(item_widgets[i], XtNcallback, 
 		    (XtCallbackProc) items[i].callback, client_data );
@@ -195,7 +215,7 @@ Widget *createButtons(ButtonList buttons, Cardinal n_buttons, Widget parent,
   for (i = 0; i < n_buttons; i++) {
     button_widgets[i] = XtVaCreateManagedWidget(buttons[i].button_name,
       		commandWidgetClass, parent,
-		XtNlabel, translate(buttons[i].button_label),
+		XtNlabel, _(buttons[i].button_label),
 		(char *)0);
 
     XtAddCallback(button_widgets[i], XtNcallback, 
@@ -252,7 +272,7 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
   /* Find width of label */
   label_args[3].value = (XtArgVal) 0;
   for (i=0; i<n_questions; i++) {
-	qlabel = translate(questions[i].label);
+	qlabel = _(questions[i].label);
     l = XTextWidth(resources.label_font, qlabel, 
 		   strlen(qlabel));
     if (l > label_args[3].value)
@@ -260,7 +280,7 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
   }
 
   for (i = 0; i<n_questions; i++) {
-	qlabel = translate(questions[i].label);
+	qlabel = _(questions[i].label);
     label_args[0].value = (XtArgVal) bitmap_widget;
     label_args[1].value = (XtArgVal) vert;
     label_args[2].value = (XtArgVal) qlabel;
@@ -277,7 +297,7 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
     text_args[2].value = (XtArgVal) questions[i].value;
     text_args[3].value = (XtArgVal) questions[i].length;
     vert = questions[i].widget = XtCreateManagedWidget("text", 
-	   	textfieldWidgetClass, form_widget,
+	   	mwTextfieldWidgetClass, form_widget,
 		text_args, XtNumber(text_args));
   }
 
@@ -328,7 +348,9 @@ void grayOut(Widget w)
 
 void tick(Widget w)
 {
-  XtVaSetValues(w, XtNleftBitmap, (XtArgVal) bm[TICK_BM], NULL);
+  load_icons();
+
+  XtVaSetValues(w, XtNleft_icon, dot_bm, NULL);
 }
 
 /*****************************************************************************/
@@ -340,7 +362,9 @@ void tick(Widget w)
 
 void noTick(Widget w)
 {
-  XtVaSetValues(w, XtNleftBitmap, (XtArgVal) bm[NOTICK_BM], NULL);
+  load_icons();
+
+  XtVaSetValues(w, XtNleft_icon, blank_bm, NULL);
 }
 
 /*****************************************************************************/
