@@ -1,5 +1,6 @@
 /*
  *  Copyright (c) 1996 Andr Hentz
+ *  Copyright (c) 1998-2003 Ulric Eriksson
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -83,9 +84,10 @@ int main(int argc, char* argv[])
 
 	common_init("Runcmd %s. No Warranty");
 
-	toplevel=XtVaAppInitialize(&app,"Runcmd",NULL,0,
+	toplevel=XtVaOpenApplication(&app,"Runcmd",NULL,0,
 				   (Cardinal*)&argc,argv,
-				   fallback_resources,NULL);
+				   fallback_resources,
+				   mwApplicationShellWidgetClass, NULL);
 
 	dpy = XtDisplay(toplevel);
 	MwHighlightInit(toplevel);
