@@ -1,6 +1,6 @@
 /*
    Pathetic Writer
-   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-2000  Ulric Eriksson <ulric@siag.nu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 
 typedef struct pw_ui {
         Widget viewport,        /* really a form */
-		rulerframe,	/* a sunken frame for the ruler */
                 ruler,		/* a canvas */
                 grid,           /* cells */
                 hscroll,        /* horizontal scrollbar */
@@ -55,6 +54,7 @@ extern Boolean convert_proc(Widget, Atom *, Atom *, Atom *,
 extern void lose_ownership_proc(Widget, Atom *);
 extern void requestor_callback(Widget, XtPointer, Atom *, Atom *,
                         XtPointer, unsigned long *, int *);
+extern void transfer_done_proc(Widget, Atom *, Atom *);
 
 /* from input.c */
 extern void DialogDoneAction(Widget, XEvent *, String *, Cardinal *);
